#!/usr/bin/env python
import ursgal
import os
import shutil


class kojak_1_5_3( ursgal.UNode ):
    """
    Kojak UNode
    Parameter options at http://www.kojak-ms.org/param/index.html

    Reference:
    Hoopmann MR, Zelter A, Johnson RS, Riffle M, Maccoss MJ, Davis TN, Moritz RL (2015) Kojak: Efficient analysis of chemically cross-linked protein complexes. J Proteome Res 14: 2190-198

    Note:

        Kojak has to be installed manually at the moment! Use folder name:
        'kojak_1_5_3' in the resources folder.

    """
    META_INFO = {
        'edit_version'                : 1.00,
        'name'                        : 'Kojak',
        'version'                     : '1.5.3',
        'release_date'                : '2015-5-1',
        'engine_type' : {
            'cross_link_search_engine' : True,
        },
        'input_extensions'            : ['.mzML', '.mzXML'],
        'output_extensions' : ['.kojak.txt', '.pep.xml', '.perc.inter.txt', \
            '.perc.intra.txt', '.perc.loop.txt', '.perc.single.txt'],
        'create_own_folder'           : True,
        'distributable'               : False,
        'in_development'              : False,
        'include_in_git'              : None,
        'utranslation_style'          : 'kojak_style_1',
        'engine' : {
            'linux' : {
                '64bit' : {
                    'exe'            : 'kojak',
                    'url'            : '',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                },
            },
            'win32' : {
                '64bit' : {
                    'exe'            : 'kojak',
                    'url'            : '',
                    'zip_md5'        : '',
                    'additional_exe' : [],
                }
            },
        },
        'citation' : \
            'Hoopmann MR, Zelter A, Johnson RS, Riffle M, Maccoss MJ, Davis '\
            'TN, Moritz RL (2015) Kojak: Efficient analysis of chemically '\
            'cross-linked protein complexes. J Proteome Res 14: 2190-198',
    }


    def __init__(self, *args, **kwargs):
        super(kojak_1_5_3, self).__init__(*args, **kwargs)
        pass

    def preflight( self ):
        '''
        Formatting the command line via self.params

        '''
        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        input_file = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )
        if input_file.lower().endswith('.mzml') or \
            input_file.lower().endswith('.mzml.gz'):
            self.params['translations']['mzml_input_file'] = input_file
        elif input_file.lower().endswith('.mgf'):
            self.params['translations']['mzml_input_file'] = \
                self.meta_unodes['ucontroller'].get_mzml_that_corresponds_to_mgf( input_file )
            self.print_info(
                'Kojak cannot read .mgf files.'
                'the corresponding mzML file {0} will be used instead.'.format(
                    os.path.abspath(self.params['translations']['mzml_input_file'])
                ),
                caller = "INFO"
            )
        else:
            raise Exception('Kojak input spectrum file must be in mzML or MGF format!')


        # remap modifications to adapt to kojak format
        self.params['translations']['kojak_fix_modifications'] = ''
        self.params['translations']['kojak_opt_modifications'] = ''
        for mod_type, mod_list in self.params['mods'].items():
            mod_dict_key = 'kojak_{0}_modifications'.format(mod_type)
            # if len(mod_list) > 0:
            #     self.p[]
            if mod_type == 'fix':
                param_name = 'fixed_modification'
            else:
                param_name = 'modification'
            for mod_dict in mod_list:
                self.params['translations'][mod_dict_key] += '{0} = {1} {2}\n'.format(
                    param_name,
                    mod_dict['aa'],
                    mod_dict['mass']
                )
        self.params['translations']['formatted_cross_link'] = ''
        self.params['translations']['formatted_mono_link'] = ''
        if len(self.params['translations']['cross_link_definition']) > 0:
            for cross_link in self.params['translations']['cross_link_definition']:
                self.params['translations']['formatted_cross_link'] += 'cross_link = {0}\n'.format(cross_link)

        if len(self.params['translations']['mono_link_definition']) > 0:
            for mono_link in self.params['translations']['mono_link_definition']:
                if mono_link not in [None, '']:
                    self.params['translations']['formatted_mono_link'] += 'mono_link = {0}\n'.format(mono_link)

        for ion in ['a', 'b', 'c', 'x', 'y', 'z']:
            if ion in self.params['translations']['score_ion_list']:
                self.params['translations']['ion_series_{0}'.format(ion.upper())] = 1
            else:
                self.params['translations']['ion_series_{0}'.format(ion.upper())] = 0

        # building command_list !

        templates = self.format_templates( )
        config_file_path = os.path.join(
            self.params['output_dir_path'],
            'Kojak.conf'
        )
        with open( config_file_path, 'w') as out:
            print(
                templates['Kojak.conf'],
                file = out
            )
            self.print_info('wrote input file {0}'.format( config_file_path ))
        self.params['command_list'] =[
            self.exe,
            '{0}'.format(config_file_path)
        ]

        return self.params

    def postflight( self ):
        '''
        Move the result files to the Kojak folder, since the output files can
        not be specified manually.
        '''
        # kojak_extensions = [
        #     '.kojak.txt',
        #     '.pep.xml',
        #     '.perc.inter.txt',
        #     '.perc.intra.txt',
        #     '.perc.loop.txt',
        #     '.perc.single.txt',
        # ]
        for extension in self.META_INFO['all_extensions']:
            org_path = os.path.join(
                self.params['input_dir_path'],
                '{0}{1}'.format(
                    self.params['file_root'],
                    extension
                )
            )
            new_path = os.path.join(
                self.params['output_dir_path'],
                '{0}_kojak_{1}{2}'.format(
                    self.params['file_root'],
                    self.META_INFO['version'].replace('.', '_'),
                    extension
                )
            )
            if os.path.exists(org_path):
                shutil.move(
                    org_path,
                    new_path
                )

        pass

    def format_templates( self ):
        '''
        Returns formatted input files as a dict.

        The standard parametern file is used and adjustes.

        Returns:
            dict: keys are the names of the parametern template file

        '''
        templates = {
            'Kojak.conf' : '''\
# Kojak version 1.5.3 parameter file
# Please see online documentation at:
# http://www.kojak-ms.org/param

# All parameters are separated from their values by an equals sign ('=')
# Anything after a '#' will be ignored for the remainder of the line.


#
# Computational resources
#
threads     =   {cpus} #Increase to use more cores/nodes


#
# Data input files: include full path if not in current working directory
#
database            =   {database}
export_percolator   =   {kojak_export_percolator}
export_pepXML       =   {kojak_export_pepxml}
MS_data_file        =   {mzml_input_file}
percolator_version  =   {kojak_percolator_version}
# output_file       =   {output_file_incl_path}


#
# Parameters used to described the data being input to Kojak
#
enrichment      =   {kojak_enrichment}       #Values between 0 and 1 to describe 18O APE.
                            #For example, 0.25 equals 25 APE.
instrument      =   {instrument}       #Values are: 0=Orbitrap, 1=FTICR (such as Thermo LTQ-FT)
MS1_centroid    =   {ms1_is_centroided}       #0=no, 1=yes
MS2_centroid    =   {ms2_is_centroided}        #0=no, 1=yes
MS1_resolution  =   {ms1_resolution}    #Resolution at 400 m/z, value ignored if data are
                            #already centroided
MS2_resolution  =   {ms2_resolution}    #Resolution at 400 m/z, value ignored if data are
                            #already centroided


#
# Cross-link and mono-link masses allowed. May have more than one of each parameter.
#
# Format for cross_link is [amino acids] [amino acids] [mass mod] [identifier]
# Format for mono_link is [amino acids] [mass mod]
# One or more amino acids (uppercase only!!) can be specified for each linkage moiety
# Use lowercase 'n' or 'c' to indicate protein N-terminus or C-terminus
#
# cross_link  =   nK  nK  138.0680742 BS3
# mono_link   =   nK  156.0786
# cross_link = C G -18.010565 Thioester_loss_of_water
{formatted_cross_link}
{formatted_mono_link}


#
# Fixed modifications. Add as many as necessary.
#
#fixed_modification  =   C   57.02146
{kojak_fix_modifications}


#
# Differential modifications. Add as many as necessary. Uppercase only!
# n = protein N-terminus, c = protein C-terminus
#
# If more than one modification is possible for an amino acid,
# list all modifications on separate lines
#
#modification            =       M       15.9949
{kojak_opt_modifications}

diff_mods_on_xl         =       {kojak_diff_mods_on_xl}
max_mods_per_peptide    =       {max_num_mods}
mono_links_on_xl        =       {kojak_mono_links_on_xl}


#
# Digestion enzyme rules.
#
# See http://www.kojak-ms.org/param/enzyme.html
#
enzyme  =       {enzyme}


#
# Scoring algorithm parameters
#
# fragment_bin_offset and fragment_bin_size influence algorithm precision and memory usage.
# They should be set appropriately for the data analyzed.
# For ion trap ms/ms:  1.0005 size, 0.4 offset
# For high res ms/ms:    0.03 size, 0.0 offset
#

fragment_bin_offset     =   {kojak_fragment_bin_offset}     #between 0.0 and 1.0
fragment_bin_size       =   {kojak_fragment_bin_size}   #in Thomsons
ion_series_A            =   {score_a_ions}
ion_series_B            =   {score_b_ions}
ion_series_C            =   {score_c_ions}
ion_series_X            =   {score_x_ions}
ion_series_Y            =   {score_y_ions}
ion_series_Z            =   {score_z_ions}

#
# Additional parameters used in Kojak analysis
#
decoy_filter            =   {decoy_tag}   #identifier for all decoys in the database.
                                    #Default value is "random" (without quotes)
isotope_error           =   {precursor_isotope_range}       #account for errors in precursor peak identification.
                                    #Searches this number of isotope peak offsets.
                                    #Values are 0,1,or 2.
max_miscleavages        =   {max_missed_cleavages}       #number of missed trypsin cleavages allowed
max_peptide_mass        =   {precursor_max_mass}  #largest allowed peptide mass in Daltons
min_peptide_mass        =   {precursor_min_mass}   #lowest allowed peptide mass in Daltons
max_spectrum_peaks      =   {max_accounted_observed_peaks}       #top N peaks to use during analysis. 0 uses all peaks.
ppm_tolerance_pre       =   {precursor_mass_tolerance_plus}    #mass tolerance on precursor when searching
prefer_precursor_pred   =   {kojak_prefer_precursor_pred}       #prefer precursor mono mass predicted by
                                    #instrument software.
                                    #  0 = ignore previous predictions
                                    #  1 = use only previous predictions
                                    #  2 = supplement predictions with additional analysis
spectrum_processing     =   {kojak_spectrum_processing}       #0=no, 1=yes
top_count               =   {kojak_top_count}     #number of top scoring single peptides to combine
                                    #in relaxed analysis
truncate_prot_names     =   {kojak_truncate_prot_names}       #Max protein name character to export, 0=off
turbo_button            =   {kojak_turbo_button}       #Generally speeds up analysis. Special cases cause reverse
                                    #effect, thus this is allowed to be disabled. 0=off
        '''.format(
            **self.params['translations']
            )
        }
        return templates



#!/usr/bin/env python3
# encoding: utf-8
'''

Test the filter_csv function for xtandem engine

'''
import ursgal
import csv
import os


R = ursgal.UController()


TESTS = [
    {
        'csv_filter_rules' : [ ['Calc m/z','gte',1100],['Calc m/z','lte',1400]  ],
        'number_of_expected_lines': 4,
    },
    {
        'csv_filter_rules' : [ ['Modifications','contains','Carbamidomethyl'] ],
        'number_of_expected_lines': 9,
    },
    {
        'csv_filter_rules' : [ ['Modifications','contains_not','Carbamidomethyl'] ],
        'number_of_expected_lines': 1,
    },
    {
        'csv_filter_rules' : [ ['Retention Time (s)','gt',1793.82605] ], #last RT
        'number_of_expected_lines': 0,
    },
    {
        'csv_filter_rules' : [ ['Retention Time (s)','lt',1793.82605] ], #last RT
        'number_of_expected_lines': 9,
    },
    {
        'csv_filter_rules' : [ ['Sequence','contains','T'] ],
        'number_of_expected_lines': 4,
    },
    {
        'csv_filter_rules' : [ ['Is decoy','equals','FALSE'] ], # Excel style
        'number_of_expected_lines': 10,
    },
    {
        'csv_filter_rules' : [ ['Is decoy','equals','TRUE'] ], # Excel style
        'number_of_expected_lines': 0,
    },
    {
        'csv_filter_rules' : [
            ['Modifications','contains','Carbamidomethyl'],
            ['PEP','lte',7.121E-05]
        ],
        'number_of_expected_lines': 1,
    },
    {
        'csv_filter_rules' : [
            ['Modifications','regex','Carbamidomethyl\:2']
        ],
        'number_of_expected_lines': 6,
    },
    {
        'csv_filter_rules' : [
            ['Spectrum Title','regex','.25\d*.']
        ],
        'number_of_expected_lines': 8,
    },
    {
        'csv_filter_rules' : [
            ['Sequence','regex','C[T|D]']
        ],
        'number_of_expected_lines': 6,
    },
    {
        'csv_filter_rules' : [
            ['Modifications','contains_element_of_list',['Oxidation', 'Carbamidomethyl']]
        ],
        'number_of_expected_lines': 9,
    },
]


filter_csv_main = R.unodes['filter_csv_1_0_0']['class'].import_engine_as_python_function()

input_csv = os.path.join(
    'tests',
    'data',
    'test_BSA1_omssa_2_1_9_unified_filter_csv_test.csv'
)


def compare_filter_test():
    for test_id, test_dict in enumerate(TESTS):
        yield compare_filter, test_dict


def compare_filter( test_dict ):
    output_csv = filter_csv_main(
        input_file     = input_csv,
        output_file    = os.path.join(
            'tests',
            'data',
            'test_BSA1_omssa_2_1_9_unified_filter_csv_test_filtered.csv'
        ),
        filter_rules   = test_dict['csv_filter_rules'],
    )
    opened_output_csv = open(output_csv, 'r')
    counter = 0
    for line_dict in csv.DictReader(opened_output_csv):
        counter += 1
    opened_output_csv.close()
    assert counter == test_dict['number_of_expected_lines']

    os.remove(output_csv)


if __name__ == '__main__':
    print(__doc__)
    for test_id, test_dict in enumerate(TESTS):
        compare_filter(test_dict)



#!/usr/bin/env python3
# encoding: utf-8
'''

Test the unify_csv function for msgfplus engine

'''
import ursgal
import csv
import pickle
import os

modifications = [
    'C,fix,any,Carbamidomethyl',  # Carbamidomethylation
    'Y,opt,any,Phospho',
    '*,opt,Prot-C-term,Amidated',
    'P,opt,any,Oxidation',
]

R = ursgal.UController(
    params = {
        'modifications' : modifications
    }    
)
R.map_mods()

scan_rt_lookup = pickle.load(
    open(
        os.path.join(
            'tests',
            'data',
            '_test_ursgal_lookup.pkl')
        ,
        'rb'
    )
)

unify_csv_main = R.unodes['unify_csv_1_0_0']['class'].import_engine_as_python_function()
input_csv = os.path.join(
    'tests',
    'data',
    'msgfplus_v9979',
    'test_BSA1_msgfplus_v9979.csv'
)
output_csv = os.path.join(
    'tests',
    'data',
    'msgfplus_v9979',
    'test_BSA1_msgfplus_v9979_unified.csv'
)
unify_csv_main(
    input_file     = input_csv,
    output_file    = output_csv,
    scan_rt_lookup = scan_rt_lookup,
    params = {
        'translations' : {
            'decoy_tag': 'decoy_',
            'enzyme' : 'KR;C;P',
            'semi_enzyme' : False,
            'database': os.path.join(
                'tests',
                'data',
                'BSA.fasta'
            ),
            'protein_delimiter'              : '<|>',
            'psm_merge_delimiter'            : ';',
            'keep_asp_pro_broken_peps'       : True,
            'precursor_mass_tolerance_minus' : 5,
            'precursor_mass_tolerance_plus'  : 5,
            'precursor_isotope_range'        : "0,1",
            'max_missed_cleavages'           : 2,
            'rounded_mass_decimals'          : 3,
            'use_pyqms_for_mz_calculation'   : False,
            'aa_exception_dict' : {
                'J' : {
                    'original_aa' : ['L','I'],
                },
                'O' : {
                    'original_aa' : ['K'],
                    'unimod_name' : 'Methylpyrroline',
                },
            },
        },
        'label' : '15N',
        'mods' : R.params['mods'],
    },
    search_engine  = 'msgfplus_v9979',
)

ident_list = [ ]
for line_dict in csv.DictReader(open(output_csv, 'r')):
    ident_list.append( line_dict )


def unify_msgfplus_test():
    for test_id, test_dict in enumerate(ident_list):
        yield unify_msgfplus, test_dict


def unify_msgfplus( test_dict ):
    assert 'uCalc m/z' in test_dict.keys()
    assert 'index=' not in test_dict['Spectrum ID']

    for key in [
            'Retention Time (s)',
            'Spectrum ID',
            'Modifications',
            'Spectrum Title',
            'Is decoy'
        ]:
        test_value = test_dict[key]
        expected_value = test_dict['Expected {0}'.format(key)]
        if key == 'Retention Time (s)':
            test_value     = round(float(test_value), 4)
            expected_value = round(float(expected_value), 4)

        assert test_value == expected_value, '''
  Unexpected value in column "{0}":
    test value:     {1}
    expected value: {2}
            '''.format(key, test_value, expected_value)


if __name__ == '__main__':
    print(__doc__)
    for test_id, test_dict in enumerate(ident_list):
        unify_msgfplus(test_dict)



#!/usr/bin/env python
# encoding: utf-8
"""

    Ursgal MappOrs


    :copyright: (c) 2014 by C. Fufezan, S. Schulze
    :licence: BSD, see LISCENSE for more details

"""
from __future__ import absolute_import
import sys
import os
import xml.etree.ElementTree as ET
import xml.dom.minidom as xmldom
import ursgal


class UnimodMapper( object ):
    '''
    UnimodMapper class that creates lookup to the unimod.xml and
    userdefined_unimod.xml found located in ursgal/kb/ext and
    offers several helper methods described below :

    '''
    def __init__( self ):
        self.data_list = self._parseXML()
        self.mapper    = self._initialize_mapper()

    def _parseXML(self, xmlFile = None):
        if xmlFile is None:
            unimodXML = os.path.normpath(
                os.path.join(
                    os.path.dirname(__file__),
                    'resources',
                    'platform_independent',
                    'arc_independent',
                    'ext',
                    'unimod.xml'
                ))
            if unimodXML is None:
                print("No unimod.xml file found.")
                sys.exit(1)
            userdefined_unimodXML = os.path.normpath(
                os.path.join(
                    os.path.dirname(__file__),
                    'resources',
                    'platform_independent',
                    'arc_independent',
                    'ext',
                    'userdefined_unimod.xml'
                ))
            xmlFiles = [unimodXML, userdefined_unimodXML] 
        else:
            xmlFiles = [xmlFile]
        data_list = []
        for xmlFile in xmlFiles:
            if os.path.exists( xmlFile ):
                unimodXML = ET.iterparse(
                    open( xmlFile, 'r', encoding='utf-8'),
                    events = (b'start', b'end')
                )
                collect_element = False
                for event, element in unimodXML:
                    if event == b'start':
                        if element.tag.endswith('}mod'):
                            tmp = {
                                'unimodID' : element.attrib['record_id'],
                                'unimodname' : element.attrib['title'],
                                'element' : {}
                            }
                        elif element.tag.endswith('}delta'):
                            collect_element = True
                            tmp[ 'mono_mass' ] = float(
                                element.attrib[ 'mono_mass' ]
                            )
                        elif element.tag.endswith('}element'):
                            if collect_element is True:
                                number = int(element.attrib['number'])
                                if number != 0:
                                    tmp['element'][ element.attrib['symbol'] ] = \
                                        number
                        else:
                            pass
                    else:
                        # end element
                        if element.tag.endswith('}delta'):
                            collect_element = False
                        elif element.tag.endswith('}mod'):
                            data_list.append(tmp)
                        else:
                            pass

            else:
                continue
        return data_list

    def _initialize_mapper(self):
        '''set up the mapper, generates the index dict'''
        mapper = {}
        for index, unimod_data_dict in enumerate(self.data_list):
            for key, value in unimod_data_dict.items():
                if key == 'element':

                    MAJORS = ['C', 'H']
                    hill_notation = ''
                    for major in MAJORS:
                        if major in unimod_data_dict[key].keys():
                            hill_notation += '{0}({1})'.format(
                                major,
                                unimod_data_dict[key][major]
                            )
                    for symbol, number in sorted(unimod_data_dict[key].items()):
                        if symbol in MAJORS:
                            continue
                        hill_notation += '{0}({1})'.format(
                            symbol,
                            number
                        )
                    if hill_notation not in mapper.keys():
                        mapper[ hill_notation ] = []
                    mapper[ hill_notation ].append( index )
                elif key == 'mono_mass':
                    if value not in mapper.keys():
                        mapper[ value ] = []
                    mapper[ value ].append( index )
                else:
                    if value not in mapper.keys():
                        mapper[ value ] = index
        return mapper

    # name 2 ....
    def name2mass(self, unimod_name):
        '''
        Converts unimod name to unimod mono isotopic mass

        Args:
            unimod_name (str):

        Returns:
            float: Unimod mono isotopic mass
        '''
        return self._map_key_2_index_2_value(unimod_name, 'mono_mass')

    def name2composition(self, unimod_name):
        '''
        Converts unimod name to unimod composition

        Args:
            unimod_name (str):

        Returns:
            dict: Unimod elemental composition
        '''
        return self._map_key_2_index_2_value(unimod_name, 'element')

    def name2id(self, unimod_name):
        '''
        Converts unimod name to unimodID

        Args:
            unimod_name (str):

        Returns:
            int: Unimod id
        '''
        return self._map_key_2_index_2_value(unimod_name, 'unimodID')

    # unimodid 2 ....
    def id2mass(self, unimod_id):
        '''
        Converts unimodID to unimod mass

        Args:
            unimod_id (int):

        Returns:
            float: Unimod mono isotopic mass
        '''
        return self._map_key_2_index_2_value(unimod_id, 'mono_mass')

    def id2composition(self, unimod_id):
        '''
        Converts unimod id to unimod composition

        Args:
            unimod_id (int):

        Returns:
            dict: Unimod elemental composition
        '''
        return self._map_key_2_index_2_value(unimod_id, 'element')

    def id2name(self, unimod_id):
        '''
        Converts unimodID to unimod name

        Args:
            unimod_id (int):

        Returns:
            str: Unimod name
        '''
        return self._map_key_2_index_2_value(unimod_id, 'unimodname')

    # mass is ambigous therefore a list is returned
    def mass2name_list(self, mass):
        '''
        Converts unimod mass to unimod name list,
            since a given mass can map to mutiple entries in the XML.

        Args:
            mass (float):

        Returns:
            list: Unimod names
        '''
        list_2_return = []
        index_list = self.mapper.get( mass, None)
        if index_list is not None:
            for index in index_list:
                list_2_return.append( self._data_list_2_value(index, 'unimodname'))
        return list_2_return

    def mass2id_list(self, mass):
        '''
        Converts unimod mass to unimod name list,
            since a given mass can map to mutiple entries in the XML.

        Args:
            mass (float):

        Returns:
            list: Unimod IDs
        '''
        list_2_return = []
        index_list = self.mapper.get( mass, None)
        if index_list is not None:
            for index in index_list:
                list_2_return.append(self._data_list_2_value(index, 'unimodID'))
        return list_2_return

    def mass2composition_list(self, mass):
        '''
        Converts unimod mass to unimod element composition list,
            since a given mass can map to mutiple entries in the XML.

        Args:
            mass (float):

        Returns:
            list: Unimod elemental compositions
        '''

        list_2_return = []
        index_list = self.mapper.get( mass, None)
        if index_list is not None:
            for index in index_list:
                list_2_return.append( self._data_list_2_value(index, 'element') )
        return list_2_return

    def composition2name_list(self, composition):
        '''
        Converts unimod composition to unimod name list,
            since a given composition can map to mutiple entries in the XML.

        Args:
            composition (dict):

        Returns:
            list: Unimod names
        '''
        list_2_return = []
        index_list = self.mapper.get( composition, None)
        if index_list is not None:
            for index in index_list:
                list_2_return.append( self._data_list_2_value(index, 'unimodname'))
        return list_2_return

    def composition2id_list(self, composition):
        '''
        Converts unimod composition to unimod name list,
            since a given composition can map to mutiple entries in the XML.

        Args:
            composition (dict):

        Returns:
            list: Unimod IDs
        '''
        list_2_return = []
        index_list = self.mapper.get( composition, None)
        if index_list is not None:
            for index in index_list:
                list_2_return.append(
                    self._data_list_2_value(index, 'unimodID')
                )
        return list_2_return

    def composition2mass(self, composition):
        '''
        Converts unimod composition to unimod monoisotopic mass,

        Args:
            composition (float):

        Returns:
            float: monoisotopic mass
        '''
        mass_2_return = None
        list_2_return = []
        index_list = self.mapper.get( composition, None)
        if index_list != None:
            for index in index_list:
                list_2_return.append( self._data_list_2_value(index, 'mono_mass') )
            assert len(set(list_2_return)) == 1, '''
            Unimod chemical composition {0}
            maps on different monoisotopic masses. This should not happen.
            '''.format( composition )
            mass_2_return = list_2_return[0]
        return mass_2_return

    def appMass2id_list(self, mass, decimal_places = 2):
        '''
        Creates a list of unimod ids for a given approximate mass

        Args:
            mass (float):

        Keyword Arguments:
            decimal_places (int): Precision with which the masses in the
                Unimod is compared to the input, i.e.
                round( mass, decimal_places )

        Returns:
            list: Unimod IDs

        Examples::
            >>> import ursgal
            >>> U = ursgal.UnimodMapper()
            >>> U.appMass2id_list(18, decimal_places=0)
            ['127', '329', '608', '1079', '1167']

        '''
        return_list = self._appMass2whatever(
            mass,
            decimal_places= decimal_places,
            entry_key='unimodID'
        )
        return return_list

    def appMass2element_list(self, mass, decimal_places = 2):
        '''
        Creates a list of element composition dicts for a given approximate mass

        Args:
            mass (float):

        Keyword Arguments:
            decimal_places (int): Precision with which the masses in the
                Unimod is compared to the input, i.e. round( mass, decimal_places )

        Returns:
            list: Dicts of elements

        Examples::
            >>> import ursgal
            >>> U = ursgal.UnimodMapper()
            >>> U.appMass2element_list(18, decimal_places=0)
            [{'F': 1, 'H': -1}, {'13C': 1, 'H': -1, '2H': 3},
                {'H': -2, 'C': -1, 'S': 1}, {'H': 2, 'C': 4, 'O': -2},
                {'H': -2, 'C': -1, 'O': 2}]


        '''
        return_list = self._appMass2whatever(
            mass,
            decimal_places= decimal_places,
            entry_key='element'
        )
        return return_list

    def appMass2name_list(self, mass, decimal_places = 2):
        '''
        Creates a list of unimod names for a given approximate mass

        Args:
            mass (float)

        Keyword Arguments:
            decimal_places (int): Precision with which the masses in the
                Unimod is compared to the input, i.e. round( mass, decimal_places )

        Returns:
            list: Unimod names

        Examples::
            >>> import ursgal
            >>> U = ursgal.UnimodMapper()
            >>> U.appMass2name_list(18, decimal_places=0)
            ['Fluoro', 'Methyl:2H(3)13C(1)', 'Xle->Met', 'Glu->Phe', 'Pro->Asp']
        '''
        return_list = self._appMass2whatever(
            mass,
            decimal_places= decimal_places,
            entry_key='unimodname'
        )
        return return_list

    def _appMass2whatever(self, mass, decimal_places=2, entry_key=None):
        return_list = []
        for entry in self.data_list:
            umass = entry['mono_mass']
            rounded_umass = round( float(umass), decimal_places )
            if abs(rounded_umass - mass) <= sys.float_info.epsilon:
                return_list.append( entry[ entry_key ] )
        return return_list

    def _map_key_2_index_2_value(self, map_key, return_key):
        ''''''
        if type(map_key) is int:
            map_key = str(map_key)
        index = self.mapper.get( map_key.strip(), None)
        if index is None:
            ursgal.UNode.print_info(
                'Cannot return {0} via map {1}'.format( \
                    return_key,
                    map_key,
                ),
                caller='WARNING'
            )
            return_value = None
        else:
            return_value = self._data_list_2_value(index, return_key)
        return return_value

    def _data_list_2_value(self, index, return_key):
        ''''''
        return self.data_list[ index ][ return_key ]

    def writeXML(self, modification_dict, xmlFile = None):
        '''
        Writes a unimod-style userdefined_unimod.xml file in 
        ursal/resources/platform_independent/arc_independent/ext

        Args:
            modification_dict (dict): dictionary containing at least
            'mass' (mass of the modification),
            'name' (name of the modificaton),
            'composition' (chmical composition of the modification as a Hill notation)
        '''
        if xmlFile == None:
            xmlFile = os.path.normpath(
                os.path.join(
                    os.path.dirname(__file__),
                    'resources',
                    'platform_independent',
                    'arc_independent',
                    'ext',
                    'userdefined_unimod.xml'
                ))
        unimod = ET.Element('{usermod}unimod')
        modifications = ET.SubElement(unimod, '{usermod}modifications')
        mod_dicts = [modification_dict]
        if os.path.exists(xmlFile):
            data_list = self._parseXML(xmlFile)
            for data_dict in data_list:
                mod_dict = {
                    'mass'  : data_dict['mono_mass'],
                    'name'  : data_dict['unimodname'],
                    'composition' : data_dict['element'],
                    'id'    : data_dict['unimodID'],
                }
                mod_dicts.insert(-1,mod_dict)

        for modification_dict in mod_dicts:
            if modification_dict.get('id', None) == None:
                modification_dict['id'] = 'u{0}'.format(len(mod_dicts))
            mod = ET.SubElement(modifications, '{usermod}mod', title = modification_dict['name'], record_id = modification_dict['id'])
            delta = ET.SubElement(mod, '{usermod}delta', mono_mass = str(modification_dict['mass']) )

            for symbol, number in modification_dict['composition'].items():
                element = ET.SubElement(delta, '{usermod}element', symbol=symbol, number=str(number) )

        tree = ET.ElementTree(unimod)
        tree.write(xmlFile, encoding = 'utf-8')
        xml = xmldom.parse(xmlFile)
        pretty_xml_as_string = xml.toprettyxml()
        outfile = open(xmlFile, 'w')
        print(pretty_xml_as_string, file = outfile)
        outfile.close()
        self._reparseXML()
        return

    def _reparseXML(self):
        self.data_list = self._parseXML()
        self.mapper    = self._initialize_mapper()

if __name__ == '__main__':
    print('Yes!')



#!/usr/bin/env python
# coding: latin1
import ursgal
import math
import csv
import bisect
import os
from collections import defaultdict as ddict
from collections import OrderedDict
import sys
from pprint import pprint
from itertools import chain

PROTON = 1.00727646677


def transform_score(score, minimum_score):
    '''Transform scores for "bigger_is_better" == False into linear and positive space'''
    minimum_score_transformed = -1 * math.log(minimum_score, 10)
    try:
        transformed_score = -1 * math.log(score, 10)
    except:
        transformed_score = minimum_score_transformed

    if transformed_score >= minimum_score_transformed:
        transformed_score = minimum_score_transformed
    return transformed_score


class percolator_3_2_1(ursgal.UNode):
    """
    Percolator 3.2.1 UNode

    q-value and posterior error probability calculation
    by a semi-supervised learning algorithm that dynamically
    learns to separate target from decoy peptide-spectrum matches (PSMs)

    Reference:
    Matthew The, Michael J. MacCoss, William S. Noble, Lukas Käll "Fast and Accurate Protein False Discovery Rates on Large-Scale Proteomics Data Sets with Percolator 3.0"

    Noe:
    Please download Percolator corresponding to your OS from: https://github.com/percolator/percolator/releases
    """
    META_INFO = {
        'engine_type': {
            'controller': False,
            'converter': False,
            'validation_engine': True,
            'search_engine': False,
            'meta_engine': False
        },
        'edit_version': 1.00,
        'name': 'percolator',
        'version': '3.2.0',
        'release_date': None,
        'output_extensions': ['.csv'],
        'output_suffix': 'percolator_3_2_1_validated',
        'input_extensions': ['.csv'],
        'create_own_folder': False,
        'citation' : 'Matthew The, Michael J. MacCoss, William S. Noble, Lukas Kall' \
            'Fast and Accurate Protein False Discovery Rates on Large-Scale Proteomics Data Sets with Percolator 3.0',
        'include_in_git': False, # True for now, but upload dat shit later on
        'distributable': False,
        'group_psms': True,
        'in_development': False,
        'utranslation_style': 'percolator_style_1',
        'engine': {
            'darwin': {
                '64bit': {
                    'exe': 'percolator_3_2_1',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
            },
            'linux': {
                '64bit': {
                    'exe': 'percolator',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
            },
            'win32': {
                '64bit': {
                    'exe': 'percolator.exe',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
                '32bit': {
                    'exe': 'percolator.exe',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
            },
        }
    }

    def __init__(self, *args, **kwargs):
        super(percolator_3_2_1, self).__init__(*args, **kwargs)
        pass

    def preflight(self):
        '''
        Formating the command line to via self.params
        '''
        PERCOLATOR_FIELDS = OrderedDict([
            (
                'SpecId', {
                    'csv_field': 'Spectrum Title',
                    'DefaultDirection': 'DefaultDirection'
                }
            ),
            (
                'Label', {
                    'csv_field': '',
                    'DefaultDirection': '-'
                }
            ),
            (
                'ScanNr', {
                    'csv_field': 'Spectrum ID',
                    'DefaultDirection': '-'
                }
            ),
            (
                'lnrSp', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                'The natural logarithm of the rank of the match based on the Sp score'
                }
            ),
            (
                'deltLCn', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                "The difference between this PSM's XCorr and the XCorr of the last-ranked \
                PSM for this spectrum, divided by this PSM's XCorr or 1, whichever is larger."
                }
            ),
            (
                'deltCn', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                    "The difference between this PSM's XCorr and the XCorr of the next-ranked \
                PSM for this spectrum, divided by this PSM's XCorr or 1, whichever is larger. \
                Note that this definition differs from that of the standard delta Cn reported \
                by SEQUEST®"
                        }
            ),
            (
                'Xcorr', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                        "The SEQUEST cross-correlation score"
                        }
            ),
            (
                'Sp', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                "The preliminary SEQUEST score."
                        }
            ),
            (
                'IonFrac', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                "The fraction of b and y ions theoretical ions matched to the spectrum"
                        }
            ),
            (
                'Mass',    {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                "The observed mass [M+H]+"
                        }
            ),
            (
                'PepLen',  {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': \
                "The length of the matched peptide, in residues"
                }
            ),
            (
                'Charge1', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge2', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge3', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge4', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge5', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge6', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge7', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge8', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge9', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'Charge10', {
                    'csv_field': '',
                    'DefaultDirection': 0
                }
            ),
            (
                'enzN', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': "Is the peptide preceded by an enzymatic (tryptic) site?"
                }
            ),
            (
                'enzC', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': "Does the peptide have an enzymatic (tryptic) C-terminus?"
                }
            ),
            (
                'enzInt', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': "Number of missed internal enzymatic (tryptic) sites"
                }
            ),
            (
                'lnNumSP', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': "The natural logarithm of the number of database peptides within the \
                    specified precursor range"
                }
            ),
            (
                'dM', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': "The difference between the calculated and observed mass"
                }
            ),
            (
                'absdM', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': "The absolute value of the difference between the calculated and \
                    observed mass"
                }
            ),
            (
                'Peptide', {
                    'csv_field': '',
                    'DefaultDirection': 0,
                    'description': ""
                }
            ),
            (
                'Proteins', {
                    'csv_field': 'Protein ID',
                    'DefaultDirection': 0,
                    'description': ""
                }
            ),
        ])

        self.params['translations']['csv_input_file'] = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )

        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )

        # this file will contain the decoys only (we need it for fdr calculations and such)
        self.params['translations']['decoy_output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            "decoysOnly_" + self.params['output_file']
        )

        self.params['translations']['percolator_in'] = \
            '{output_file_incl_path}.tsv'.format(
                **self.params['translations']
        )

        self.params['translations']['percolator_out'] = \
            '{output_file_incl_path}.psms'.format(**self.params['translations'])

        # writing the Percolator input file (tab separated format)
        o = open(self.params['translations']['percolator_in'], 'w')
        writer = csv.DictWriter(
            o,
            list(PERCOLATOR_FIELDS.keys()),
            delimiter='\t',
            extrasaction='ignore'
        )
        writer.writeheader()

        self.params['translations']['percolator_decoy_out'] = \
            '{decoy_output_file_incl_path}.psms'.format(**self.params['translations'])

        self.params['translations']['protein_output_target'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file'].strip('.csv') + '_prot_inf_targets.csv'
        )
        self.params['translations']['protein_output_decoy'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file'].strip('.csv') + '_prot_inf_decoys.csv'
        )

        self.params['translations']['peptide_output_target'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file'].strip('.csv') + '_pep_inf_targets.csv'
        )
        self.params['translations']['peptide_output_decoy'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file'].strip('.csv') + '_pep_inf_decoys.csv'
        )

        self.params['command_list'] = [
            self.exe,
            '--only-psms',
            '{percolator_in}'.format(**self.params['translations']),
            '--results-psms',
            '{percolator_out}'.format(**self.params['translations']),
            '--decoy-results-psms',
            '{percolator_decoy_out}'.format(**self.params['translations']),
        ]

        if self.params['translations']['percolator_post_processing'] == 'mix-max':
            self.params['command_list'].append(
                '-y'
            )
        elif self.params['translations']['percolator_post_processing'] == 'tdc':
            self.params['command_list'].append(
                '-Y'
            )
        else:
            raise Exception('post processing method must be either mix-max or tdc')

        if self.params['infer_proteins'] is True:
            self.params['command_list'].remove('--only-psms')
            self.params['command_list'] += [
                '--picked-protein',
                self.params['database'],
                '-P',
                '{decoy_tag}'.format(**self.params),
                '--results-proteins',
                '{protein_output_target}'.format(**self.params['translations']),
                '--decoy-results-proteins',
                '{protein_output_decoy}'.format(**self.params['translations']),
                '--results-peptides',
                '{peptide_output_target}'.format(**self.params['translations']),
                '--decoy-results-peptides',
                '{peptide_output_decoy}'.format(**self.params['translations']),
                '--protein-enzyme',
                self.params['enzyme']
            ]

        last_search_engine = self.get_last_search_engine(
            history=self.stats['history']
        )

        self.params['_score_list'] = self.generating_score_list()
        minimum_score = None
        bigger_scores_better = self.UNODE_UPARAMS['bigger_scores_better']['uvalue_style_translation'][last_search_engine]
        if bigger_scores_better is False:
            for p, _score in enumerate(self.params['_score_list']):
                if _score <= 0:
                    # jumping over truly zero or negative values ...
                    continue
                else:
                    s = -1 * math.log(_score, 10)
                    if minimum_score is None:
                        minimum_score = _score
                # OMSSA HACK OMG PLEASE HELP US ALL
                if "OMSSA" in last_search_engine.upper():
                    fo = transform_score(_score, minimum_score)
                    if fo < 100:
                        minimum_score = _score
                        break
                else:
                    break

        for n, spectrum_title in enumerate(self.params['grouped_psms'].keys()):
            best_score = self.params['grouped_psms'][spectrum_title][0][0]
            worst_score = self.params['grouped_psms'][spectrum_title][-1][0]

            if bigger_scores_better is False:
                best_score = transform_score(best_score, minimum_score)
                worst_score = transform_score(worst_score, minimum_score)

            for m, (score, line_dict) in enumerate(
                    self.params['grouped_psms'][spectrum_title]):
                t = {}
                if bigger_scores_better is True:
                    rank_of_score = bisect.bisect_right(
                        self.params['_score_list'],
                        score
                    )
                else:
                    rank_of_score = bisect.bisect_left(
                        self.params['_score_list'],
                        score
                    )
                    rank_of_score = len(self.params['_score_list']) - rank_of_score
                #
                # t['lnrSp'] = math.log( 1 + rank_of_score )
                # t['Sp'] = rank_of_score
                #

                charge      = float(line_dict['Charge'])
                exp_mz      = float(line_dict['Exp m/z'])
                t['Mass']   = (exp_mz * charge) - (charge - 1) * PROTON
                #
                t['Xcorr'] = score
                if bigger_scores_better is False:
                    t['Xcorr'] = transform_score(t['Xcorr'], minimum_score)

                t['PepLen'] = len(line_dict['Sequence'])
                t['Charge{Charge}'.format(**line_dict)] = 1

                normalization = t['Xcorr']
                if t['Xcorr'] < 1:
                    normalization = 1
                if m == len(self.params['grouped_psms'][spectrum_title]) - 1:
                    # last entry
                    deltLCn = 0
                    deltCn = 0
                else:
                    deltLCn = (t['Xcorr'] - worst_score) / normalization
                    next_score = self.params['grouped_psms'][spectrum_title][m + 1][0]
                    if bigger_scores_better is False:
                        next_score = transform_score(next_score, minimum_score)

                    deltCn = (t['Xcorr'] - next_score) / normalization
                t['deltCn'] = deltCn
                t['deltLCn'] = deltLCn
                if line_dict['Is decoy'].upper() == 'TRUE':
                    t['Label'] = -1
                else:
                    t['Label'] = 1

                # if self.params['translations']['decoy_tag'] in line_dict['proteinacc_start_stop_pre_post_;']:
                #     # bug mzIdentML msgf+ convert
                #     t['Label'] = -1

                # this - is - sparta (or, if you like mzIdentML) ...
                # splitted = line_dict['proteinacc_start_stop_pre_post_;'].split('_')
                # aka http://imgur.com/WjiX9
                pre_aa = []
                for prot_pre_aa in line_dict['Sequence Pre AA'].split(self.params['translations']['protein_delimiter']):
                    for p_pre_aa in prot_pre_aa.split(';'):
                        pre_aa.append(p_pre_aa)
                post_aa = []
                for prot_post_aa in line_dict['Sequence Post AA'].split(self.params['translations']['protein_delimiter']):
                    for p_post_aa in prot_post_aa.split(';'):
                        post_aa.append(p_post_aa)
                allowed_aa = set(self.params['translations']['enzyme'].split(
                    ';')[0] + '-'
                )
                cleavage_site = self.params['translations']['enzyme'].split(
                    ';'
                )[1]
                inhibitor_aa = self.params['translations']['enzyme'].split(
                    ';'
                )[2]
                final_pre_aa = pre_aa[0]
                final_post_aa = post_aa[0]
                t['enzN'] = 0
                t['enzC'] = 0
                if cleavage_site == 'C':
                    for i, aa in enumerate(pre_aa):
                        if aa in allowed_aa  \
                            or line_dict['Sequence Start'] in ['1', '2']:
                            t['enzN'] = 1
                            final_pre_aa = aa
                            final_post_aa = post_aa[i]
                        if line_dict['Sequence'][-1] in allowed_aa\
                            or post_aa == '-':
                            t['enzC'] = 1
                elif cleavage_site == 'N':
                    for i, aa in enumerate(post_aa):
                        if aa in allowed_aa:
                            t['enzC'] = 1
                            final_post_aa = aa
                            final_pre_aa = pre_aa[i]
                        if line_dict['Sequence'][0] in allowed_aa\
                            or line_dict['Sequence Start'] in ['1', '2']:
                            t['enzN'] = 1

                t['enzInt'] = 0
                for aa in line_dict['Sequence'][:-1]:
                    if aa in allowed_aa:
                        t['enzInt'] += 1

                t['dM'] = float(line_dict['Calc m/z']) - float(line_dict['Exp m/z'])
                t['absdM'] = abs(t['dM'])

                mods = line_dict['Modifications']
                if mods.strip() == '':
                    t['Peptide'] = '{0}.{Sequence}.{1}'.format(
                        sorted(pre_aa)[0],
                        sorted(post_aa)[0],
                        **line_dict
                    )
                else:
                    t['Peptide'] = '{0}.{Sequence}#{1}.{2}'.format(
                        sorted(pre_aa)[0],
                        mods,
                        sorted(post_aa)[0],
                        **line_dict
                    )
                # although peptides without mod ill have # at their end,
                # mapping further down will still work ...
                for per_key in PERCOLATOR_FIELDS.keys():
                    mapped_key = PERCOLATOR_FIELDS[per_key]['csv_field']
                    if 'Charge' in per_key:
                        if per_key not in t.keys():
                            t[per_key] = 0
                    if mapped_key != '':
                        ##### Does this break anything??
                        # replace spaces so percolator does not fuck up everything ...
                        space_name = line_dict[mapped_key].strip()
                        no_space_name = line_dict[mapped_key].strip().replace(
                            ' ', '_'
                        )
                        t[per_key] = no_space_name

                writer.writerow(t)
        o.close()

        # marking temporary files for deletion:
        self.created_tmp_files += [
            self.params['translations']['decoy_output_file_incl_path'],
            # self.params['translations']['percolator_in'],
            '{output_file_incl_path}.psms'.format(
                **self.params['translations']
            ),
            '{output_file_incl_path}.peptides'.format(
                **self.params['translations']
            ),
        ]

    def postflight(self):
        '''
        read the output and merge in back to the ident csv
        '''

        potential_buggy_percolator_output = self.params['translations']['percolator_out'] + '.psms'
        if os.path.exists(potential_buggy_percolator_output):
            # print('Renaming: \n{percolator_out}.psms ->> {percolator_out}'.format(
            #     **self.params['translations']
            # ))
            os.rename(
                self.params['translations']['output_file_incl_path'] + '.psms.psms',
                self.params['translations']['output_file_incl_path'] + '.psms'
            )
            os.rename(
                self.params['translations']['decoy_output_file_incl_path'] + '.psms.psms',
                self.params['translations']['decoy_output_file_incl_path'] + '.psms'
            )
            os.rename(
                self.params['translations']['output_file_incl_path'] + '.psms.peptides',
                self.params['translations']['output_file_incl_path'] + '.peptides'
            )

        s2l = {
            'target': ddict(list),
            'decoy' : ddict(list)
        }
        for pkey, p_out in [('target', 'percolator_out'), ('decoy', 'percolator_decoy_out')]:

            percolator_output_dict_reader = csv.DictReader(
                open(
                    self.params['translations'][p_out],
                    'r'
                ),
                delimiter='\t'
            )
            for line_dict in percolator_output_dict_reader:

                peptide = line_dict['peptide'].split('.')[1]
                psmid_pep_key = (
                    line_dict['PSMId'],
                    peptide,
                )
                if psmid_pep_key not in s2l[pkey].keys():
                    s2l[pkey][psmid_pep_key] = line_dict

        opened_file = open(self.params['translations']['csv_input_file'], 'r')
        csv_input = csv.DictReader(row for row in opened_file if not row.startswith('#'))

        if "PEP" not in csv_input.fieldnames and "q-value" not in csv_input.fieldnames:
            csv_input.fieldnames += ['PEP', 'q-value']
        csv_kwargs = {}

        if sys.platform == 'win32':
            csv_kwargs['lineterminator'] = '\n'
        else:
            csv_kwargs['lineterminator'] = '\r\n'

        csv_output = csv.DictWriter(
            open(self.params['translations']['output_file_incl_path'], 'w'),
            csv_input.fieldnames + ['Protein q-value', 'Protein PEP', 'Protein Group'],
            **csv_kwargs
        )

        if self.params['infer_proteins'] is True:
        # if os.path.exists(self.params['translations']['protein_output_target']) and \
        #  os.path.exists(self.params['translations']['protein_output_decoy']):
            with open(self.params['translations']['protein_output_target']) as fin1,\
             open(self.params['translations']['protein_output_decoy']) as fin2:
                target_proteins_reader = csv.DictReader(
                    fin1,
                    delimiter='\t'
                )
                decoy_proteins_reader = csv.DictReader(
                    fin2,
                    delimiter='\t'
                )

                protein_fdr_lookup = {
                    line['ProteinId']: {
                        'q-value': line['q-value'],
                        'PEP': line['posterior_error_prob'],
                        'ID': line['ProteinGroupId']
                    } for line in target_proteins_reader
                }
                for line in decoy_proteins_reader:
                    protein_fdr_lookup[line['ProteinId']] = {
                        'q-value': line['q-value'],
                        'PEP': line['posterior_error_prob'],
                        'ID': line['ProteinGroupId']
                    }

        csv_output.writeheader()
        for line_dict in csv_input:
            psm_type = "target"
            if line_dict['Is decoy'].upper() == 'TRUE':
                psm_type = "decoy"

            if self.params['infer_proteins'] is True:
                line_dict['Protein q-value'] = protein_fdr_lookup.get(
                    line_dict['Protein ID'].replace(' ', '_'),
                    {'q-value': ''}
                )['q-value']
                line_dict['Protein PEP'] = protein_fdr_lookup.get(
                    line_dict['Protein ID'].replace(' ', '_'),
                    {'PEP': ''}
                )['PEP']
                line_dict['Protein Group'] = protein_fdr_lookup.get(
                    line_dict['Protein ID'].replace(' ', '_'),
                    {'ID': ''}
                )
            if line_dict['Modifications'].strip() != '':
                seq_and_mods = '#'.join([line_dict['Sequence'], line_dict['Modifications']])
            else:
                seq_and_mods = line_dict['Sequence']
            _psmid_pep_key = (
                line_dict['Spectrum Title'],
                seq_and_mods,
            )
            if _psmid_pep_key in s2l[psm_type].keys():
                line_dict['PEP'] = s2l[psm_type][_psmid_pep_key]['posterior_error_prob']
                line_dict['q-value'] = s2l[psm_type][_psmid_pep_key]['q-value']
                csv_output.writerow(line_dict)
            else:
                print(
                    'Original PSM :{0} could not be found in percolator output file, most probably because PSM was filtered by percolator, (multiple peptides to one spectrum match)'.format(
                        _psmid_pep_key
                    )
                )

    def generating_score_list(self):
        scores = []
        for k, v in self.params['grouped_psms'].items():
            for score, line_dict in v:
                scores.append(score)
        # this will depnd on the scoring scheme ...
        # e.g. OMSSA smaller better - XTandem bigger better
        scores.sort()  # reverse=params['bigger_scores_better'])
        return scores



#!/usr/bin/env python3
'''
'''
import os
import ursgal


class sugarpy_run_1_0_0(ursgal.UNode):
    """
    SugarPy - discovery-driven analysis of glycan compositions from IS-CID of intact glycopeptides.
    """
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'SugarPy',
        'version'            : '1.0.0',
        'release_date'       : None,
        'engine_type'        : {
            'misc_engine' : True
        },
        'utranslation_style' : 'sugarpy_run_style_1',
        'citation'           : '',
        'input_extensions'   : ['.csv'],
        'output_extensions'  : ['.csv'],
        'create_own_folder'  : True,
        'in_development'     : True,
        'include_in_git'     : False,
        'distributable'      : False,
        'engine'             : {
            'platform_independent' : {
                'arc_independent' : {
                    'exe': 'sugarpy_run_1_0_0.py',
                },
            },
        },
    }

    def __init__(self, *args, **kwargs):
        """."""
        super(sugarpy_run_1_0_0, self).__init__(*args, **kwargs)

    def _execute(self):
        """Run SugarPy on a given .mzML file based on identified
        peptides from an evidences.csv

        Translated Ursgal parameters are passed to the SugarPy main function.
        """

        self.time_point(tag='execution')

        main = self.import_engine_as_python_function()

        output_file = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )

        input_file = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )

        # mzml_files = []

        # for input_file_dict in self.params['input_file_dicts']:
        #     mzml_files.append(
        #         os.path.join(
        #             input_file_dict['dir'],
        #             input_file_dict['file']
        #         )
        #     )

        translations = self.params['translations'][
            '_grouped_by_translated_key']

        pyqms_params = {
            'PERCENTILE_FORMAT_STRING': None,
            'M_SCORE_THRESHOLD': None,
            'ELEMENT_MIN_ABUNDANCE': None,
            'MIN_REL_PEAK_INTENSITY_FOR_MATCHING': None,
            'REQUIRED_PERCENTILE_PEAK_OVERLAP': None,
            'MINIMUM_NUMBER_OF_MATCHED_ISOTOPOLOGUES': None,
            'INTENSITY_TRANSFORMATION_FACTOR': None,
            'UPPER_MZ_LIMIT': None,
            'LOWER_MZ_LIMIT': None,
            'MZ_TRANSFORMATION_FACTOR': None,
            'REL_MZ_RANGE': None,
            'REL_I_RANGE': None,
            'INTERNAL_PRECISION': None,
            'MAX_MOLECULES_PER_MATCH_BIN': None,
            'SILAC_AAS_LOCKED_IN_EXPERIMENT': None,
            'BUILD_RESULT_INDEX': None,
            'MACHINE_OFFSET_IN_PPM': None,
            'FIXED_LABEL_ISOTOPE_ENRICHMENT_LEVELS': None,
            'MZ_SCORE_PERCENTILE': None,
        }
        sugarpy_params = {}
        sugarpy_params['charges'] = list(range(
            self.params['translations']['precursor_min_charge'],
            self.params['translations']['precursor_max_charge'] + 1
        ))

        for translated_key, translation_dict in translations.items():
            if translated_key == 'REL_MZ_RANGE':
                if self.params['translations']['ms_level'] == 1:
                    print(
                        '''
                        [ WARNING ] precursor_mass_tolerance_plus and precursor_mass_tolerance_minus
                        [ WARNING ] need to be combined for SugarPy (use of symmetric tolerance window).
                        [ WARNING ] The arithmetic mean is used.
                        '''
                    )
                    pyqms_params['REL_MZ_RANGE'] = (
                        float(self.params['translations']['precursor_mass_tolerance_plus']) +
                        float(self.params['translations'][
                              'precursor_mass_tolerance_minus'])) / 2.0
                    if self.params['translations']['precursor_mass_tolerance_unit'] == 'da':
                        pyqms_params['REL_MZ_RANGE'] = \
                            ursgal.ucore.convert_dalton_to_ppm(
                                pyqms_params['REL_MZ_RANGE'],
                                base_mz=self.params['translations']['base_mz']
                        )
                else:
                    pyqms_params['REL_MZ_RANGE'] = \
                        self.params['translations']['frag_mass_tolerance']
                    if self.params['translations']['frag_mass_tolerance_unit'] == 'da':
                        pyqms_params['REL_MZ_RANGE'] = \
                            ursgal.ucore.convert_dalton_to_ppm(
                                pyqms_params['REL_MZ_RANGE'],
                                base_mz=self.params['translations']['base_mz']
                        )
                pyqms_params['REL_MZ_RANGE'] = pyqms_params['REL_MZ_RANGE'] * 1e-6
            elif translated_key in pyqms_params.keys():
                pyqms_params[translated_key] = list(
                    translation_dict.values())[0]
            elif 'charge' in translated_key:
                continue
            elif len(translation_dict) == 1:
                sugarpy_params[translated_key] = list(
                    translation_dict.values())[0]
            else:
                print('The translatd key ', translated_key,
                      ' maps on more than one ukey, but no special rules have been defined')
                print(translation_dict)
                sys.exit(1)
        sugarpy_params['pyqms_params'] = pyqms_params
        sugarpy_params['ident_file'] = input_file
        sugarpy_params['output_file'] = output_file
        sugarpy_params['force'] = True

        out = main(**sugarpy_params)

        self.print_execution_time(tag='execution')
        return out



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import unittest
import pprint
import sys
# from ursgal import umapmaster as umama
import pprint
import os
try:
    import regex
    regex_module_installed = True
except:
    regex_module_installed = False
    


TEST_FASTA = [
    '>Protein1\n',
    'ELVISLIVES\n',
    '\n',
    '>Protein2\n',
    'KLEINERPENNER\n',
    '>Protein3\n',
    'WHYISELVIS\n',
    'HELEAVING\n',
    '>T1',
    'AAAAAAA___AAAAAAAAAA_____AAAAA__________AAAAAAAAAA',
    #----+----|----+----|----+----|----+----|----+----|
    '>Overlapping',
    'GGGGGGGGGG',
    '>GnomeChompsky',
    'MUSTACHIO',
    '>Overlap_protein',
    'OVERLAP',
    '>Overlap_protein2',
    'OVERLAP'
]

TEST_FASTA_TWO = [
    '>Ze_other_protein2\n',
    'KLEINERRETTER\n',
]


class UMapMaster(unittest.TestCase):
    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def setUp(self):
        self.uc = ursgal.UController( verbose = False )
        upapa_class = self.uc.unodes['upeptide_mapper_1_0_0']['class'].import_engine_as_python_function(
            'UPeptideMapper_v3'
        )
        self.database_path =  os.path.join(
            'tests',
            'data',
            'Test.fasta'
        )
        with open(self.database_path, 'w') as io:
            for line in TEST_FASTA:
                print(line.strip(), file=io )
        # print(upapa_class)
        self.upapa_class = upapa_class(
            fasta_database = self.database_path
        )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_purge_fasta(self):
        self.upapa_class.purge_fasta_info( 'Test.fasta' )
        self.assertFalse(
            'Test.fasta' in self.upapa_class.protein_sequences.keys()
        )
        self.assertFalse(
            'Test.fasta' in self.upapa_class.protein_indices.keys()
        )
        self.assertFalse(
            'Test.fasta' in self.upapa_class.peptide_2_protein_mappings.keys()
        )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_fasta_id_parsed_and_available(self):
        input_fastas = []
        for line in TEST_FASTA:
            if line.startswith('>'):
                input_fastas.append( line[1:].strip() )

        self.assertEqual(
            sorted( self.upapa_class.protein_sequences['Test.fasta'].keys() ),
            sorted( input_fastas )
        )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_peptide_mapping_1(self):
        self.upapa_class.map_peptides(
            [ 'ELVIS' ],
            'Test.fasta'
        )
        self.assertEqual(
            [ (d['id'], d['start']) for d in self.upapa_class.peptide_2_protein_mappings['Test.fasta']['ELVIS'] ],
            [
                ('Protein1', 1),
                ('Protein3', 6),
            ]
        )

        self.upapa_class.map_peptides(
            [ 'MUSTACHIO' ],
            'Test.fasta'
        )
        self.assertEqual(
            [ (d['id'], d['start']) for d in self.upapa_class.peptide_2_protein_mappings['Test.fasta']['MUSTACHIO'] ],
            [
                ('GnomeChompsky', 1),
            ]
        )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_peptide_mapping_2(self):
        maps = self.upapa_class.map_peptides( ['VISHE'], 'Test.fasta')['VISHE']
        self.assertEqual( len(maps), 1 )
        for mapping in maps:
            self.assertEqual(
                'Protein3',
                mapping['id']
            )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_incremental_cache_buildups(self):
        '''

        '''
        self.upapa_class.cache_database(
            self.database_path,
            'Test.fasta'
        )
        # map with one parsed fasta
        maps = self.upapa_class.map_peptides( ['KLEINER'], 'Test.fasta')['KLEINER']
        # print(maps)
        # print(maps)
        self.assertEqual( len(maps), 1)
        tmp_database_path =  os.path.join(
            'tests',
            'data',
            'tmp_Test.fasta'
        )
        with open(tmp_database_path, 'w') as io:
            for line in TEST_FASTA_TWO:
                print(line.strip(), file=io )

        self.upapa_class.cache_database(
            tmp_database_path,
            'tmp_Test.fasta'
        )
        # sys.exit(1)
        # print(self.upapa_class.automatons)
        # print(s)
        maps = self.upapa_class.map_peptides(
            ['KLEINER'],
            'Test.fasta'
        )['KLEINER']
        # print(self.upapa_class.peptide_2_protein_mappings)
        # print(maps)
        self.assertEqual(
            len(maps),
            1
         )
        # sys.exit(1)
        self.assertEqual(
            len(
                self.upapa_class.map_peptides(
                    ['KLEINER'],
                    'tmp_Test.fasta'
                )['KLEINER']
            ),
            1
         )

        os.remove(tmp_database_path)
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_peptide_gt_word_but_not_continous(self):
        self.assertEqual(
            self.upapa_class.map_peptides(
                ['WHYELVIS'],
                'Test.fasta'
            )['WHYELVIS'],
            []
        )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_multiple_occurrence_in_one_seq(self):
        maps = self.upapa_class.map_peptides( ['AAAAAAAAAA'], 'Test.fasta')['AAAAAAAAAA']
        self.assertEqual( len(maps), 2 )
        sorted_maps = sorted( maps, key= lambda x: x['start'])
        self.assertEqual(
            sorted( maps, key= lambda x: x['start']),
            sorted(
                [
                    {
                        'pre'   : '_',
                        'post'  : '_',
                        'start' : 11,
                        'end'   : 20,
                        'id'    : 'T1'
                    },
                    {
                        'pre'   : '_',
                        'post'  : '-',
                        'start' : 41,
                        'end'   : 50,
                        'id'    : 'T1'
                    }

                ],
                key = lambda x: x['start']
            )
        )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_multiple_occurrence_with_opverlap_in_one_seq(self):
        maps = self.upapa_class.map_peptides(
            ['GGGGGGG'],
            'Test.fasta'
        )['GGGGGGG']
        # print(maps)
        self.assertEqual( len(maps), 4 )
        self.assertEqual(
            sorted([ m['start'] for m in maps] ),
            [1,2,3,4]
        )
        maps = self.upapa_class.map_peptides(
            ['GGGG'],
            'Test.fasta'
        )['GGGG']
        # print(maps)
        self.assertEqual( len(maps), 7 )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_overlap_produces_no_match(self):
        maps = self.upapa_class.map_peptides(
            ['LAPOVER'],
            'Test.fasta'
        )['LAPOVER']
        self.assertEqual( len(maps), 0 )
        return

    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_start_and_end_of_sequence(self):
        maps = self.upapa_class.map_peptides(
            ['OVERLAP'],
            'Test.fasta'
        )['OVERLAP']
        # print(maps)
        for map_dict in maps:
            self.assertEqual( map_dict['post'], '-' )
            self.assertEqual( map_dict['pre'], '-' )
        return


    @unittest.skipIf(
        sys.platform == 'win32',
        'pyahocorasick not installed via pip in Windows'
    )
    def test_end_of_sequence_and_fasta(self):
        maps = self.upapa_class.map_peptides(
            ['LAP'],
            'Test.fasta'
        )['LAP']
        # print(maps)
        for map_dict in maps:
            self.assertEqual( map_dict['post'], '-' )
        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import glob
import csv
from collections import defaultdict as ddict
import os
import shutil


def main():
    '''
    Example script to do a precursor mass tolerance parameter sweep.

    usage:
        ./bsa_precursor_mass_tolerance_example.py


    '''
    precursor_mass_tolerance_list = [
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        9,
        10,
    ]

    engine_list = [
        'xtandem_vengeance'
    ]

    R = ursgal.UController(
        profile='LTQ XL low res',
        params={
            'database': os.path.join(
                os.pardir,
                'example_data',
                'BSA.fasta'
            ),
            'modifications': [
                'M,opt,any,Oxidation',        # Met oxidation
                'C,fix,any,Carbamidomethyl',  # Carbamidomethylation
                '*,opt,Prot-N-term,Acetyl'    # N-Acteylation
            ],
        }
    )

    mzML_file = os.path.join(
        os.pardir,
        'example_data',
        'BSA_precursor_mass_tolerance_example',
        'BSA1.mzML'
    )
    if os.path.exists(mzML_file) is False:

        R.params['http_url'] = 'http://sourceforge.net/p/open-ms/code/HEAD/tree/OpenMS/share/OpenMS/examples/BSA/BSA1.mzML?format=raw'

        R.params['http_output_folder'] = os.path.dirname(mzML_file)
        R.fetch_file(
            engine='get_http_files_1_0_0'
        )
        try:
            shutil.move(
                '{0}?format=raw'.format(mzML_file),
                mzML_file
            )
        except:
            shutil.move(
                '{0}format=raw'.format(mzML_file),
                mzML_file
            )
    # Convert mzML to MGF outside the loop, so this step is not repeated in
    # the loop
    mgf_file = R.convert(
        input_file=mzML_file,
        engine='mzml2mgf_1_0_0'
    )

    for engine in engine_list:
        for precursor_mass_tolerance in precursor_mass_tolerance_list:

            R.params['precursor_mass_tolerance_minus'] = precursor_mass_tolerance
            R.params['precursor_mass_tolerance_plus'] = precursor_mass_tolerance

            R.params['prefix'] = '{0}_precursor_mass_tolerance_'.format(
                precursor_mass_tolerance
            )

            unified_search_result_file = R.search(
                input_file=mgf_file,
                engine=engine,
                force=False,
            )

    collector = ddict(set)
    for csv_path in glob.glob('{0}/*/*unified.csv'.format(os.path.dirname(mzML_file))):
        for line_dict in csv.DictReader(open(csv_path, 'r')):
            collector[csv_path].add(line_dict['Sequence'])
    for csv_path, peptide_set in sorted(collector.items()):
        file_name = os.path.basename(csv_path)
        tolerance = file_name.split('_')[0]
        print(
            'Search with {0: >2} ppm precursor mass tolerance found {1: >2} peptides'.format(
                tolerance,
                len(peptide_set)
            )
        )

    return

if __name__ == '__main__':
    main()



#!/usr/bin/env python
import ursgal
import os


class plot_pygcluster_heatmap_from_csv_1_0_0( ursgal.UNode ):
    """plot_pygcluster_heatmap_from_csv_1_0_0 UNode"""
    META_INFO = {
        'edit_version'       : 1.00,
        'name'               : 'Plot pyGCluster Heatmap from CSV',
        'version'            : '1.0.0',
        'release_date'       : '2014-3-15',
        'engine_type' : {
            'visualizer' : True,
        },
        'input_extensions'   : ['.csv'],
        'output_extensions'  : ['.svg'],
        'output_suffix'      : '',
        'include_in_git'     : True,
        'in_development'     : True,
        'distributable'      : True,
        'utranslation_style' : 'heatmap_style_1',
        'engine' : {
            'platform_independent' : {
                'arc_independent' : {
                    'exe' : 'plot_pygcluster_heatmap_from_csv_1_0_0.py',
                },
            },
        },
        'citation' : \
            'Jaeger D, Barth J, Niehues A, Fufezan C (2014) pyGCluster, a '\
            'novel hierarchical clustering approach. Bioinformatics 30 896 898',
    }

    def __init__(self, *args, **kwargs):
        super(plot_pygcluster_heatmap_from_csv_1_0_0, self).__init__(*args, **kwargs)

    def _execute( self ):
        '''
        '''
        print('[ -ENGINE- ] Executing plotting ..')
        # self.time_point(tag = 'execution')
        unify_csv_main = self.import_engine_as_python_function()
        output_file = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        input_file  = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )
        unify_csv_main(
            input_file      = input_file,
            output_file     = output_file,
            params          = self.params['translations'],
        )
        # self.print_execution_time(tag='execution')
        return output_file



#!/usr/bin/env python3
# encoding: utf-8
'''

Test the unify_csv function for myrimatch engine

'''
import ursgal
import csv
import pickle
import os

modifications = [
    'C,fix,any,Carbamidomethyl',  # Carbamidomethylation
]

R = ursgal.UController(
    params = {
        'modifications' : modifications
    }    
)
R.map_mods()

scan_rt_lookup = pickle.load(
    open(
        os.path.join(
            'tests',
            'data',
            '_test_ursgal_lookup.pkl')
        ,
        'rb'
    )
)


unify_csv_main = R.unodes['unify_csv_1_0_0']['class'].import_engine_as_python_function()
input_csv = os.path.join(
    'tests',
    'data',
    'myrimatch_2_1_138',
    'test_BSA1_myrimatch_2_1_138.csv'
)
output_csv = os.path.join(
    'tests',
    'data',
    'myrimatch_2_1_138',
    'test_BSA1_myrimatch_2_1_138_unified.csv'
)
unify_csv_main(
    input_file     = input_csv,
    output_file    = output_csv,
    scan_rt_lookup = scan_rt_lookup,
    params = {
        'translations': {
            'decoy_tag': 'decoy_',
            'enzyme' : 'KR;C;P',
            'semi_enzyme' : False,
            'database': os.path.join(
                'tests',
                'data',
                'BSA.fasta'
            ),
            'protein_delimiter' : '<|>',
            'psm_merge_delimiter' : ';',
            'keep_asp_pro_broken_peps':True,
            'precursor_mass_tolerance_minus': 5,
            'precursor_mass_tolerance_plus' : 5,
            'precursor_isotope_range' : "0,1",
            'max_missed_cleavages' : 2,
            'rounded_mass_decimals' : 3,
            'use_pyqms_for_mz_calculation' : False,
            'aa_exception_dict' : {
                'J' : {
                    'original_aa' : ['L','I'],
                },
                'O' : {
                    'original_aa' : ['K'],
                    'unimod_name' : 'Methylpyrroline',
                },
            },
        },
        'label' : '15N',
        'mods' : R.params['mods'],
    },
    search_engine  = 'myrimatch_2_1_138',
)
ident_list = [ ]
for line_dict in csv.DictReader(open(output_csv, 'r')):
    ident_list.append( line_dict )


def unify_myrimtach_test():
    for test_id, test_dict in enumerate(ident_list):
        yield unify_myrimtach, test_dict


def unify_myrimtach( test_dict ):
    assert 'uCalc m/z' in test_dict.keys()
    assert 'scan=' not in test_dict['Spectrum ID']
    fields_to_eval = [
        'Retention Time (s)',
        'Spectrum ID',
        'Modifications',
        'Spectrum Title',
        'Sequence',
    ]

    for key in fields_to_eval:
        test_value = test_dict[key]
        expected_value = test_dict['Expected {0}'.format(key)]
        if key == 'Retention Time (s)':
            test_value     = round(float(test_value), 4)
            expected_value = round(float(expected_value), 4)

        assert test_value == expected_value


if __name__ == '__main__':
    print(__doc__)
    for test_id, test_dict in enumerate(ident_list):
        unify_myrimtach(test_dict)



#!/usr/bin/env python
# encoding: utf-8

import ursgal
import os
import sys
import shutil


def main(input_raw_file=None, database=None, enzyme=None):
    '''
    This script executes three steps:
        - convert .raw file using pParse
        - search the resulting .mgf file using pGlyco
        - validate results using pGlycoFDR

    Since pGlyco does not support providing a custom target-decoy database,
    a database including only target sequences must be provided.
    In this database, the N-glycosylation motif N-X-S/T needs to be replaced
    with J-X-S/T, which will be done by the pGlyco wrapper.

    usage:
        ./simple_example_pglyco_workflow.py <input_raw_file> <database> <enzyme>

    '''
    uc = ursgal.UController(
        profile='QExactive+',
        params={
            'database': database,
            'modifications': [
                'M,opt,any,Oxidation',        # Met oxidation
                'C,fix,any,Carbamidomethyl',  # Carbamidomethylation
                '*,opt,Prot-N-term,Acetyl'    # N-Acteylation
            ],
            'peptide_mapper_class_version': 'UPeptideMapper_v4',
            'enzyme': enzyme,
            'frag_mass_tolerance'       : 20,
            'frag_mass_tolerance_unit'  : 'ppm',
            'precursor_mass_tolerance_plus' : 5,
            'precursor_mass_tolerance_minus' : 5,
            'aa_exception_dict' : {},
        }
    )

    xtracted_file = uc.convert(
        input_file = input_raw_file,
        engine = 'pparse_2_0',
        # force = True,
    )

    mgf_file = uc.convert(
        input_file = input_raw_file.replace('.raw', '.idx.gz'),
        engine = 'mzml2mgf_2_0_0',
        # force = True,
    )

    search_result = uc.search_mgf(
        input_file = xtracted_file,
        engine = 'pglyco_db_2_2_0',
    )

    # converted_result = uc.convert(
    #     input_file=search_result,
    #     guess_engine = True,
    # )
        
    mapped_results = uc.execute_misc_engine(
        input_file=search_result,
        engine='upeptide_mapper',
    )

    unified_search_results = uc.execute_misc_engine(
        input_file = mapped_results,
        engine='unify_csv'
    )

    validated_file = uc.validate(
        input_file=search_result,
        engine='pglyco_fdr_2_2_0',
    )

    mapped_validated_results = uc.execute_misc_engine(
        input_file=validated_file,
        engine='upeptide_mapper',
    )

    unified_validated_results = uc.execute_misc_engine(
        input_file = mapped_validated_results,
        engine='unify_csv'
    )

    return


if __name__ == '__main__':
    main(input_raw_file=sys.argv[1], database=sys.argv[2],enzyme=sys.argv[3])



#!/usr/bin/env python3

# encoding: utf-8

import ursgal
import csv
import os


TESTS = [

    {'peptide': 'STGGAAGMLGGARSQRVVR'},
    {'peptide': 'FTMADGGSIEPQENTSGEAFVEELIR'},
    {'peptide': 'AAGGAIPHHTYAAACIDTEDVEAAAAACR'},
    {'peptide': 'AERTGGEWGVSRDGAGGGSAGGGGR'},
    {'peptide': 'QDAAVTTGTVAAAAAAAAFGGGK'},
    {'peptide': 'AMVAGGVEAGVLADGADERSAVQLGK'},
    {'peptide': 'IEGEFAAVEELDSSAGLLK'},
    {'peptide': 'EGMRPRFTSSVPGEYR'},
    {'peptide': 'AKAGASAADGTAASTNSESAEGRNSAEEK'},
    {'peptide': 'ATSTESDEIALLEK'},
    {'peptide': 'SPATGPATSGGLMSASPGNGAMVR'},
    {'peptide': 'VEEDGEELSLDEESDALAASDFSQAEPR'},
    {'peptide': 'RRGVGGGSGGSCGGGSGGQGGCGGGGAGR'},
    {'peptide': 'AAAAAAPPADETAAAAAALEAVATAAAAPAPEAAPVGR'},
    {'peptide': 'APGAGPGTTLLITDIQSSTALWELLPEDVMAK'},
    {'peptide': 'AVSRANGGGGGGAAAAANTGGAAASTSGGGAAAEGAGAAGEGGEDPAMR'},
    {'peptide': 'SSGGGGSRGAIGTGTGGANKFPQPK'},
    {'peptide': 'GLLLDLADPVIGAAWEPHISDPGLWRFLKPHLK'},
    {'peptide': 'GWLAVQALEPQAERREAPAPGPTAAVSAAAPAAADK'},
    {'peptide': 'FQAAPWTFFLRSPLTDEPVDASMCR'},
    {'peptide': 'VFQRVTVNAVDEWDEEARMHVAER'},
    {'peptide': 'LGALEDADQGFSLLGIGFDEWK'},
    {'peptide': 'CMNPVTDIAGGAEALLPSPTAPPQGSHSEVAR'},
    {'peptide': 'GGSGGGGGGGGGGDGSGGRMVPGSGGSSAQAGQGPGDSLAGAPSASAAGR'},
    {'peptide': 'QTQLEDQAEEEEAEAQAQAEK'},
    {'peptide': 'KAFLEYLAVQQEQLAREGVAELPPEQQEQLMR'},
    {'peptide': 'QLKDIEERSSAVSSGITENEEADLIEIIVISLNK'},
    {'peptide': 'LGSTSAPPSASKHGTPAGGGGSGGGR'},
    {'peptide': 'LGLFFPAGLAHTEPDRSKEPQYAQCGTR'},
    {'peptide': 'VHGAYERYPTFLVQAHYEAGRGAAAEFMGYSGR'},
    {'peptide': 'DLNEIPAAVQIILALELFSAVQDPMHLRDAKVK'},
    {'peptide': 'GSSCSGTYVSGSALDVTHTGTSGQDSSGDVWGSAAEKRSGGR'},
    {'peptide': 'LDFGMPGYLDGAKVAVEMRAGGLQLR'},
    {'peptide': 'SEVAQPVGVEGSPSALLPAGLGSLVAGK'},
    {'peptide': 'AAAAATQAGSRDPAHSLGKAAAAASLAAALLDASLLTTLHVHVPSGR'},
]

R = ursgal.UController(
    profile = 'LTQ XL low res',
    params = {
        'database': 'tests/data/test_Creinhardtii_target_decoy.fasta',
        'modifications':[
            '*,opt,Prot-N-term,Acetyl'    # N-Acteylation[]
        ]
    },
    force=True
)
IS_AVAILABLE = R.unodes['omssa_2_1_9']['available']

if IS_AVAILABLE:
    output_file = R.search(
        input_file = 'tests/data/test_Creinhardtii_QE_pH8.mzML',
        engine     = 'omssa',
        force      = True
    )
else:
    output_file = os.path.join(
        'tests',
        'data',
        'test_shipped_Creinhardtii_QE_pH8_omssa_2_1_9_unified.csv'
    )

peptides_set = set()
for line_dict in csv.DictReader(open(output_file, 'r')):
    peptides_set.add( line_dict['Sequence'] )


def compare_peptide_test():
    for test_id, test_dict in enumerate(TESTS):
        yield compare_peptide, test_dict


def compare_peptide( test_dict ):
    assert test_dict['peptide'] in peptides_set


def number_of_peptides_test():
    test_peptides = set()
    for peptide_dict in TESTS:
        test_peptides.add( peptide_dict['peptide'] )
    assert len(peptides_set) == len(test_peptides)


if __name__ == '__main__':
    print(__doc__)



#!/usr/bin/env python
# encoding: utf-8

'''

Retrieve data from ftp server

usage:
    get_ftp_files_1_0_0.py <ftp_address> <login> <password> <filter_entension>

'''

# import glob
import ftplib
from ftplib import FTP
import os
import tempfile


def main(
        ftp_url             = None,
        folder              = None,
        login               = None,
        password            = None,
        include_ext         = None,
        output_folder       = None,
        max_number_of_files = None,
        blocksize           = None
    ):
    # retrieve files via ftp
    assert ftp_url is not None, '[ -<FTP>-- ] Require ftp_url not None to run ;)'
    print(
        '[ -<FTP>-- ] Downloading files from {0}, this can take a while...'.format(
            ftp_url
        )
    )
    if include_ext is None:
        include_ext = set()
    # statinfo = os.stat( target_path )
    # 'size' : statinfo.st_size

    ftp = FTP(ftp_url.replace('ftp://', ''))
    ftp.login(
        user   = login,
        passwd = password,
    )
    if folder is not None:
        ftp.cwd( '/' + folder + '/' )
        # does not hurt, just to be sure ...

    if output_folder is None:
        output_folder = tempfile.gettempdir()

    downloaded_files = []

    def download_file(source, target, file_size ):
        print(
            '[ -<FTP>-- ] Downloading: {0} into {1} with file size of {2:1.1f} MB'.format(
                source,
                target,
                file_size / 1e6,
            )
        )
        with open(target, 'wb') as io:
            ftp.retrbinary(
                'RETR ' + source,
                io.write,
                blocksize=1024
            )

        return

    def walk_deeper( folder = None, output_root = None, downloaded_files=None):
        if folder is None:
            folder = ''
        if downloaded_files is None:
            downloaded_files = []
        for file_or_directory in ftp.nlst( folder ):
            # print( file_or_directory )
            try:
                ftp_size  = ftp.size(file_or_directory)
                is_file = True
                # this raises exeption ftplib.error_perm on peptideatlas.org
            except ftplib.error_perm:
                is_file = False
                walk_deeper(
                    file_or_directory,
                    output_root=output_root
                )
            if is_file:
                allowed_file = False
                for extension in include_ext:
                    if file_or_directory.upper().endswith(extension.upper()):
                        allowed_file = True
                        break

                # for extension in exclude_ext:
                #     if file_or_directory.upper().endswith(extension.upper()):
                #         allowed_file = False
                # this DOES not work ... :)

                if allowed_file:
                    dirname             = os.path.dirname(file_or_directory)
                    file_path_on_host   = os.path.join(
                        output_root,
                        file_or_directory
                    )
                    folder_path_on_host = os.path.join(output_root, dirname )
                    if os.path.exists(file_path_on_host):
                        if ftp_size != os.stat( file_path_on_host ).st_size:
                            print(
                                '[ -<FTP>-- ] Downloading again: {0} because download was incomplete!'.format(
                                    file_path_on_host
                                )
                            )
                            download_file(
                                file_or_directory,
                                file_path_on_host,
                                ftp_size
                            )
                        else:
                            print(
                                '[ -<FTP>-- ] File: {0} already downloaded!'.format(
                                    file_path_on_host
                                )
                            )
                            downloaded_files.append( file_path_on_host )
                    else:
                        if os.path.exists(folder_path_on_host) is False:
                            print(
                                '[ -<FTP>-- ] Created directory: {0}'.format(
                                    folder_path_on_host
                                )
                            )
                            os.makedirs( folder_path_on_host )

                        download_file(
                            file_or_directory,
                            file_path_on_host,
                            ftp_size
                        )
                        downloaded_files.append( file_path_on_host )
        return downloaded_files

    downloaded_files = walk_deeper(
        output_root      = output_folder,
        downloaded_files = downloaded_files
    )

    ftp.quit()
    return downloaded_files

if __name__ == '__main__':
    main(
        ftp_url               = 'ftp.pride.ebi.ac.uk',
        folder                = '/pride/data/archive/2013/08/PXD000278',
        include_ext           = ['.txt'],
        output_folder         = '/tmp',
        # max_number_of_files = 1,
        blocksize             = None
    )




#!/usr/bin/env python
# encoding: utf-8
"""
plot.py

by
    Stefan Schulze
    Anna Niehues
    Johannes Barth
    Till Bald
    Daniel Jaeger
    Christian Fufezan

"""
from __future__ import print_function
import os
import sys
import bisect
import math
import codecs

def main( *args, **kwargs ):
    """
    Creates a simple SVG VennDiagram
    requires 2, 3, 4 or 5 sets as arguments

    Keyword Arguments:

        output_file

        header

        label_A
        label_B
        label_C
        label_D
        label_E

        color_A:  e.g. #FF8C00
        color_B
        color_C
        color_D
        color_E

        font

    the function returns a dict with the following keys were the results
    can be accesse by e.g. dict['C-(A|B|D)']['results']

        'A&B-(C|D)'
        'C&D-(A|B)'
        'B&C-(A|D)'
        'A&B&C&D'
        'A&C-(B|D)'
        'B&D-(A|C)'
        'A&D-(B|C)'
        '(A&C&D)-B'
        '(A&B&D)-C'
        '(A&B&C)-D'
        '(B&C&D)-A'
        'A-(B|C|D)'
        'D-(A|B|C)'
        'B-(A|C|D)'
        'C-(A|B|D)'

    or for 2 or 3  or 5 VennDiagrams the appropriate combinations ...

    """

    assert len(args) >= 2, "2 Sets (A, B ) are minimal arguments ! :)"
    for _ in args:
        assert isinstance(_, set) , "Input args have to be Python sets, got ... {0}".format( type(_))

    defaultValues = {
        'output_file'            : 'VennDiagram.svg',
        'header'                 : 'ursgal Venn Diagram',
        'width'                  : 1200,
        'height'                 : 1200,
        'label_A'                : 'A',
        'label_B'                : 'B',
        'label_C'                : 'C',
        'label_D'                : 'D',
        'label_E'                : 'E',
        'cx'                     : 600,
        'cy'                     : 400,
        'label font-size header' : 31,
        'label font-size major'  : 25,
        'label font-size minor'  : 20,
        'label font-size venn'   : 20,
        'demo'                   : False,
        'font'                   : 'Helvetica',
        'stroke-width'           : 2,
        'opacity'                : 0.3
    }

    for k, v in defaultValues.items():
        if k not in kwargs.keys():
            kwargs[ k ] = v
    
    A = args[0]
    B = args[1]
    if len(args) == 2:
        C = set()
        D = set()
        E = set()
        kwargs['total-pos-cy'] = kwargs['cy'] + 220
    elif len(args) == 3:
        C = args[2]
        D = set()
        E = set()
        kwargs['total-pos-cy'] = kwargs['cy'] + 320
    elif len(args) == 4:
        C = args[2]
        D = args[3]
        E = set()
        kwargs['total-pos-cy'] = kwargs['cy'] + 320
    elif len(args) == 5:
        C = args[2]
        D = args[3]
        E = args[4]
        kwargs['total-pos-cy'] = kwargs['cy'] + 410
    else:
        exit('WooT?')

    kwargs['total_n'] = len(A | B | C | D | E)

    vdLen = len( args )
    vdTypeSpecific = {
        2 : {   'A' : { 'color'         : '#e41a1c',
                        'cx'            : kwargs['cx'] - 80,
                        'cy'            : kwargs['cy'],
                        'rx'            : 170,
                        'ry'            : 170,
                        'rot'           : 0,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'] - 280,
                        'text-pos-y'    : kwargs['cy'] - 170,
                        # 'set'           : sets['A'],
                        },
                'B' : { 'color'         : '#377eb8',
                        'cx'            : kwargs['cx'] + 80,
                        'cy'            : kwargs['cy'],
                        'rx'            : 170,
                        'ry'            : 170,
                        'rot'           : 0,
                        'text-anchor'   : 'start',
                        'text-pos-x'    : kwargs['cx'] + 280,
                        'text-pos-y'    : kwargs['cy'] - 170,
                        }
                },
        3 : {   'A' : { 'color'         : '#e41a1c',
                        'cx'            : kwargs['cx'] - 80,
                        'cy'            : kwargs['cy'] + 80,
                        'rx'            : 220,
                        'ry'            : 130,
                        'rot'           : 50,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'] - 300,
                        'text-pos-y'    : kwargs['cy'] - 70,
                        },
                'B' : { 'color'         : '#377eb8',
                        'cx'            : kwargs['cx'],
                        'cy'            : kwargs['cy'] - 25,
                        'rx'            : 220,
                        'ry'            : 130,
                        'rot'           : 90,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'],
                        'text-pos-y'    : kwargs['cy'] - 310,
                        },
                'C' : { 'color'         : '#4daf4a',
                        'cx'            : kwargs['cx'] + 80,
                        'cy'            : kwargs['cy'] + 80,
                        'rx'            : 220,
                        'ry'            : 130,
                        'rot'           : -50,
                        'text-anchor'   : 'start',
                        'text-pos-x'    : kwargs['cx'] + 300,
                        'text-pos-y'    : kwargs['cy'] - 70,
                        }
                },
        4 : {   'A' : { 'color'         : '#e41a1c', # '#FF8C00',
                        'cx'            : kwargs['cx'] - 80,
                        'cy'            : kwargs['cy'] + 80,
                        'rx'            : 220,
                        'ry'            : 130,
                        'rot'           : 50,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'] - 300,
                        'text-pos-y'    : kwargs['cy'] - 70,
                        },
                'B' : { 'color'         : '#377eb8', # '#FF1493',
                        'cx'            : kwargs['cx'],
                        'cy'            : kwargs['cy'],
                        'rx'            : 220,
                        'ry'            : 130,
                        'rot'           : +50,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'] - 200,
                        'text-pos-y'    : kwargs['cy'] - 200,
                        },
                'C' : { 'color'         : '#4daf4a', # '#7D26CD',
                        'cx'            : kwargs['cx'],
                        'cy'            : kwargs['cy'],
                        'rx'            : 220,
                        'ry'            : 130,
                        'rot'           : -50,
                        'text-anchor'   : 'start',
                        'text-pos-x'    : kwargs['cx'] + 200,
                        'text-pos-y'    : kwargs['cy'] - 200,
                        },
                'D' : { 'color'         : '#984ea3', # '#00CED1',
                        'cx'            : kwargs['cx'] + 80,
                        'cy'            : kwargs['cy'] + 80,
                        'rx'            : 220,
                        'ry'            : 130,
                        'rot'           : -50,
                        'text-anchor'   : 'start',
                        'text-pos-x'    : kwargs['cx'] + 300,
                        'text-pos-y'    : kwargs['cy'] - 70,
                        }
                },
        5 : {   'A' : { 'color'         : '#e41a1c',
                        'cx'            : kwargs['cx'] - 70,
                        'cy'            : kwargs['cy'] + 20,
                        'rx'            : 280,
                        'ry'            : 160,
                        'rot'           : 14,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'] - 360,
                        'text-pos-y'    : kwargs['cy'] - 60,
                },
                'B' : { 'color'         : '#377eb8',
                        'cx'            : kwargs['cx'] + 10,
                        'cy'            : kwargs['cy'] ,
                        'rx'            : 280,
                        'ry'            : 160,
                        'rot'           : 86,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'] + 0,
                        'text-pos-y'    : kwargs['cy'] - 320,

                },
                'C' : { 'color'         : '#4daf4a',
                        'cx'            : kwargs['cx'] + 50,
                        'cy'            : kwargs['cy'] + 65,
                        'rx'            : 280,
                        'ry'            : 160,
                        'rot'           : 158,
                        'text-anchor'   : 'start',
                        'text-pos-x'    : kwargs['cx'] + 330,
                        'text-pos-y'    : kwargs['cy'] - 60,
                },
                'D' : { 'color'         : '#984ea3',
                        'cx'            : kwargs['cx'] + 5,
                        'cy'            : kwargs['cy'] + 125,
                        'rx'            : 280,
                        'ry'            : 160,
                        'rot'           : 230,
                        'text-anchor'   : 'start',
                        'text-pos-x'    : kwargs['cx'] + 210,
                        'text-pos-y'    : kwargs['cy'] + 350,
                },
               'E' : { 'color'         : '#ff7f00',
                        'cx'            : kwargs['cx'] - 75,
                        'cy'            : kwargs['cy'] + 105,
                        'rx'            : 280,
                        'ry'            : 160,
                        'rot'           : 302,
                        'text-anchor'   : 'end',
                        'text-pos-x'    : kwargs['cx'] - 270,
                        'text-pos-y'    : kwargs['cy'] + 350,
                }
            },
    }
    for k, v in vdTypeSpecific[ vdLen ].items():
        vdTypeSpecific[ vdLen ][ k ]['label']   = kwargs['label_{0}'.format(k)]
        vdTypeSpecific[ vdLen ][ k ]['setSize'] = len( eval(k) )
        vdTypeSpecific[ vdLen ][ k ]['label font-size major'] = \
            kwargs['label font-size major']
        vdTypeSpecific[ vdLen ][ k ]['label font-size minor'] = \
            kwargs['label font-size minor']
        vdTypeSpecific[ vdLen ][ k ]['label font-size venn']  = \
            kwargs['label font-size venn']

        if k not in kwargs.keys():
            kwargs[ k ] = v
    #updating color:
    for setIdentifier in ['A','B','C','D','E']:
        if 'color_{0}'.format( setIdentifier ) in kwargs.keys():
            vdTypeSpecific[ vdLen ][ setIdentifier ]['color'] = kwargs[ 'color_{0}'.format( setIdentifier ) ]

    io = open( kwargs['output_file'], 'w' )
    print("""
<svg xmlns="http://www.w3.org/2000/svg" version="1.1"
width="{width}" height="{height}"
style="position:relative; top:0; left:0; z-index:-1;">
<title>{header}</title>
<g font-family="{font}" >
<text transform="translate({cx} 40)" font-size="{label font-size header}" text-anchor="middle">{header}</text>
<text transform="translate({cx} {total-pos-cy})"  font-size="{label font-size major}" text-anchor="middle">Total</text>
<text transform="translate({cx} {y2})"  font-size="{label font-size minor}" text-anchor="middle" font-style="italic">n = {total_n}</text>
</g>    """.format(
                    y2 = kwargs['total-pos-cy'] + 30,
                    **kwargs
            ), file = io)

    for setKey in sorted( vdTypeSpecific[ len(args) ].keys() ):
        # Why do we have multiple storages ?
        kwargs[ setKey ]['opacity'] =  kwargs['opacity']
        kwargs[ setKey ]['stroke-width'] =  kwargs['stroke-width']
        # -----
        print('''
        <ellipse rx="{rx}" ry="{ry}" transform="translate({cx} {cy}) rotate({rot})" style="fill:{color};fill-opacity:{opacity};stroke:{color};stroke-width:{stroke-width}" />
        '''.format( **kwargs[ setKey ]), file = io )

    print('<g font-family="{font}" >'.format( **kwargs), file = io)

    for setKey in sorted( vdTypeSpecific[ len(args) ].keys() ):
        print('''
        <text transform="translate({text-pos-x} {text-pos-y})"  font-size="{label font-size major}" text-anchor="{text-anchor}">{label}</text>
        <text transform="translate({text-pos-x} {y2})"  font-size="{label font-size minor}" text-anchor="{text-anchor}" font-style="italic">n = {setSize}</text>
        '''.format(
                    y2 = kwargs[ setKey ]['text-pos-y']+30,
                    **kwargs[ setKey ]
                    # **kwargs
            ),  file = io )
    print('</g>', file = io)

    if len(args) == 2:
        returnDict = {
            'A&B'       : {
                'value'  : 'A.B',
                'x'      : kwargs['cx'] ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
            'B-A'       : {
                'value'  : 'B.C',
                'x'      : kwargs['cx'] + 170 ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
            'A-B'       : {
                'value'  : 'A.C',
                'x'      : kwargs['cx'] - 170 ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
        }
    elif len(args) == 3:
        returnDict = {
            'A&B-C'     : {
                'value'  : 'A.B',
                'x'      : kwargs['cx'] - 90  ,
                'y'      : kwargs['cy']        ,
                'results': None, 
            },
            'B&C-A'     : {
                'value'  : 'B.C',
                'x'      : kwargs['cx'] + 90  ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
            'A&C-B'     : {
                'value'  : 'A.C',
                'x'      : kwargs['cx']       ,
                'y'      : kwargs['cy'] + 240  ,
                'results': None,
            },
            'A&B&C'     : {
                'value'  : 'A.B.C',
                'x'      : kwargs['cx']       ,
                'y'      : kwargs['cy'] + 100  ,
                'results': None,
            },
            'A-(B|C)'   : {
                'value'  : 'A',
                'x'      : kwargs['cx'] - 190 ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
            'C-(A|B)'   : {
                'value'  : 'C',
                'x'      : kwargs['cx'] + 190 ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
            'B-(A|C)'   : {
                'value'  : 'B',
                'x'      : kwargs['cx']       ,
                'y'      : kwargs['cy'] - 140  ,
                'results': None,
            },
        }
    elif len(args) == 4:
        returnDict = {
            'A&B-(C|D)' : {
                'value'  : 'A.B',
                'x'      : kwargs['cx'] - 140 ,
                'y'      : kwargs['cy'] - 70   ,
                'results': None,
            },
            'C&D-(A|B)' : {
                'value'  : 'C.D',
                'x'      : kwargs['cx'] + 140 ,
                'y'      : kwargs['cy'] - 70   ,
                'results': None,
            },
            'B&C-(A|D)' : {
                'value'  : 'B.C',
                'x'      : kwargs['cx']       ,
                'y'      : kwargs['cy'] - 70   ,
                'results': None,
            },
            'A&B&C&D'   : {
                'value'  : 'A.B.C.D',
                'x'      : kwargs['cx']       ,
                'y'      : kwargs['cy'] + 100  ,
                'results': None,
            },
            'A&C-(B|D)' : {
                'value'  : 'A.C',
                'x'      : kwargs['cx'] - 125 ,
                'y'      : kwargs['cy'] + 135  ,
                'results': None,
            },
            'B&D-(A|C)' : {
                'value'  : 'B.D',
                'x'      : kwargs['cx'] + 125 ,
                'y'      : kwargs['cy'] + 135  ,
                'results': None,
            },
            'A&D-(B|C)' : {
                'value'  : 'A.D',
                'x'      : kwargs['cx']       ,
                'y'      : kwargs['cy'] + 240  ,
                'results': None,
            },
            '(A&C&D)-B' : {
                'value'  : 'A.C.D',
                'x'      : kwargs['cx'] - 55  ,
                'y'      : kwargs['cy'] + 170  ,
                'results': None,
            },
            '(A&B&D)-C' : {
                'value'  : 'A.B.D',
                'x'      : kwargs['cx'] + 55  ,
                'y'      : kwargs['cy'] + 170  ,
                'results': None,
            },
            '(A&B&C)-D' : {
                'value'  : 'A.B.C',
                'x'      : kwargs['cx'] - 95  ,
                'y'      : kwargs['cy'] + 30   ,
                'results': None,
            },
            '(B&C&D)-A' : {
                'value'  : 'B.C.D',
                'x'      : kwargs['cx'] + 95  ,
                'y'      : kwargs['cy'] + 30   ,
                'results': None,
            },
            'A-(B|C|D)' : {
                'value'  : 'A',
                'x'      : kwargs['cx'] - 200 ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
            'D-(A|B|C)' : {
                'value'  : 'D',
                'x'      : kwargs['cx'] + 200 ,
                'y'      : kwargs['cy']        ,
                'results': None,
            },
            'B-(A|C|D)' : {
                'value'  : 'B',
                'x'      : kwargs['cx'] - 95  ,
                'y'      : kwargs['cy'] - 140  ,
                'results': None,
            },
            'C-(A|B|D)' : {
                'value'  : 'C',
                'x'      : kwargs['cx'] + 95  ,
                'y'      : kwargs['cy'] - 140  ,
                'results': None,
            },
        }
    elif len(args) == 5:
        returnDict = {
            'A-(B|C|D|E)'       : {'value' : 'A',         'x': kwargs['cx'] - 270  , 'y' : kwargs['cy'] - 30   , 'results': None },
            'D-(A|B|C|E)'       : {'value' : 'D',         'x': kwargs['cx'] + 130  , 'y' : kwargs['cy'] + 300  , 'results': None },
            'B-(A|C|D|E)'       : {'value' : 'B',         'x': kwargs['cx']        , 'y' : kwargs['cy'] - 200  , 'results': None },
            'C-(A|B|D|E)'       : {'value' : 'C',         'x': kwargs['cx'] + 240  , 'y' : kwargs['cy'] - 10   , 'results': None },
            'E-(B|C|D|A)'       : {'value' : 'E',         'x': kwargs['cx'] - 180  , 'y' : kwargs['cy'] + 300  , 'results': None },
            'A&B-(C|D|E)'       : {'value' : 'A.B',       'x': kwargs['cx'] - 110  , 'y' : kwargs['cy'] - 120  , 'results': None },
            'C&D-(A|B|E)'       : {'value' : 'C.D',       'x': kwargs['cx'] + 170  , 'y' : kwargs['cy'] + 170  , 'results': None },
            'B&C-(A|D|E)'       : {'value' : 'B.C',       'x': kwargs['cx'] + 135  , 'y' : kwargs['cy'] - 70   , 'results': None },
            'E&C-(A|D|B)'       : {'value' : 'C.E',       'x': kwargs['cx'] - 175  , 'y' : kwargs['cy'] + 190  , 'results': None },
            'B&E-(A|D|C)'       : {'value' : 'B.E',       'x': kwargs['cx'] + 40   , 'y' : kwargs['cy'] - 120  , 'results': None },
            'A&E-(B|D|C)'       : {'value' : 'A.E',       'x': kwargs['cx'] - 230  , 'y' : kwargs['cy'] + 90   , 'results': None },
            'E&D-(A|C|B)'       : {'value' : 'D.E',       'x': kwargs['cx'] - 50   , 'y' : kwargs['cy'] + 275  , 'results': None },
            'A&C-(B|D|E)'       : {'value' : 'A.C',       'x': kwargs['cx'] + 185  , 'y' : kwargs['cy'] + 80   , 'results': None },
            'B&D-(A|C|E)'       : {'value' : 'B.D',       'x': kwargs['cx'] + 50   , 'y' : kwargs['cy'] + 260  , 'results': None },
            'A&D-(B|C|E)'       : {'value' : 'A.D',       'x': kwargs['cx'] - 175  , 'y' : kwargs['cy'] - 60   , 'results': None },
            '(A&C&D)-(B|E)'     : {'value' : 'A.C.D',     'x': kwargs['cx'] + 175  , 'y' : kwargs['cy'] + 125  , 'results': None },
            '(A&B&D)-(C|E)'     : {'value' : 'A.B.D',     'x': kwargs['cx'] - 130  , 'y' : kwargs['cy'] - 90   , 'results': None },
            '(A&B&C)-(D|E)'     : {'value' : 'A.B.C',     'x': kwargs['cx'] + 150  , 'y' : kwargs['cy'] + 10   , 'results': None },
            '(B&C&D)-(A|E)'     : {'value' : 'B.C.D',     'x': kwargs['cx'] + 90   , 'y' : kwargs['cy'] + 205  , 'results': None },
            '(B&D&E)-(A|C)'     : {'value' : 'B.D.E',     'x': kwargs['cx'] - 15   , 'y' : kwargs['cy'] + 260  , 'results': None },
            '(E&B&C)-(A|D)'     : {'value' : 'B.C.E',     'x': kwargs['cx'] + 85   , 'y' : kwargs['cy'] - 95   , 'results': None },
            '(C&D&E)-(A|B)'     : {'value' : 'C.D.E',     'x': kwargs['cx'] - 105  , 'y' : kwargs['cy'] + 220  , 'results': None },
            '(A&B&E)-(D|C)'     : {'value' : 'A.B.E',     'x': kwargs['cx'] - 10   , 'y' : kwargs['cy'] - 105  , 'results': None },
            '(A&D&E)-(B|C)'     : {'value' : 'A.D.E',     'x': kwargs['cx'] - 180  , 'y' : kwargs['cy'] + 10   , 'results': None },
            '(A&E&C)-(D|B)'     : {'value' : 'A.C.E',     'x': kwargs['cx'] - 200  , 'y' : kwargs['cy'] + 130  , 'results': None },
            '(A&B&C&D)-E'       : {'value' : 'A.B.C.D',   'x': kwargs['cx'] + 120  , 'y' : kwargs['cy'] + 140  , 'results': None },
            '(A&B&C&E)-D'       : {'value' : 'A.B.C.E',   'x': kwargs['cx'] + 75   , 'y' : kwargs['cy'] - 60   , 'results': None },
            '(A&B&D&E)-C'       : {'value' : 'A.B.D.E',   'x': kwargs['cx'] - 110  , 'y' : kwargs['cy'] - 55   , 'results': None },
            '(A&C&D&E)-B'       : {'value' : 'A.C.D.E',   'x': kwargs['cx'] - 170  , 'y' : kwargs['cy'] + 90   , 'results': None },
            '(B&C&D&E)-A'       : {'value' : 'B.C.D.E',   'x': kwargs['cx'] - 20   , 'y' : kwargs['cy'] + 220  , 'results': None },
            'A&B&C&D&E'         : {'value' : 'A.B.C.D.E', 'x': kwargs['cx'] - 10   , 'y' : kwargs['cy'] + 50   , 'results': None },
            }
    else:
        print('''
            NOT SURE WHAT YOU WANT TO DO WITH MORE THAN 5 SETS ...
        ''')
    # exit(1)
    for k in returnDict.keys():
        r                          = eval( k )
        if kwargs['demo'] == False:
            returnDict[ k ]['value']   = len( r )
        returnDict[ k ]['results'] = r
    print('<g font-family="{font}" font-size="{label font-size venn}" >'.format(**kwargs), file = io)
    for labelDict in returnDict.values():
        print('<text transform="translate({x} {y})" text-anchor="middle" stroke="#777777" stroke-width="0.5" >{value}</text>'.format(**labelDict),file=io)
    print('</g>', file=io)
    print("</svg>", file = io)
    print('Saved VennDiagram as {output_file}'.format(**kwargs ))
    return returnDict

if __name__ == '__main__':
    main(
        set( [ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9              ] ),
        set( [    1,                 7, 8, 9, 10         ] ),
        set( [    1,    3,                   10, 15      ] ),
        set( [    1,                                 100 ] ),
        set( [ 0, 1,    3,    5,    7,               100 ] ),
        demo = False
    )



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import os
import shutil


def main():
    '''
    Simple example script to demonstrate conversion for mzML to mgf file
    conversion

    '''
    uc = ursgal.UController(
        profile = 'LTQ XL low res',
        params  = {},
    )

    mzML_file = os.path.join(
        os.pardir,
        'example_data',
        'simple_mzml_to_mgf_conversion',
        'BSA1.mzML'
    )
    if os.path.exists(mzML_file) is False:
        uc.params['http_url'] = 'http://sourceforge.net/p/open-ms/code/HEAD/tree/OpenMS/share/OpenMS/examples/BSA/BSA1.mzML?format=raw'
        uc.params['http_output_folder'] = os.path.dirname(mzML_file)
        uc.fetch_file(
            engine='get_http_files_1_0_0'
        )
        try:
            shutil.move(
                '{0}?format=raw'.format(mzML_file),
                mzML_file
            )
        except:
            shutil.move(
                '{0}format=raw'.format(mzML_file),
                mzML_file
            )

    mgf_file = uc.convert(
        input_file = mzML_file,  # from OpenMS example files
        engine     = 'mzml2mgf_1_0_0'
    )

if __name__ == '__main__':
    print(__doc__)
    main()



#!/usr/bin/env python3
# encoding: utf-8

import ursgal
import os
import sys
import shutil


def main():
    '''
    Executes a search with OMSSA, XTandem and MS-GF+ on the BSA1.mzML
    input_file

    usage:
        ./simple_example_search.py

    Note:
        Myrimatch does not work with this file.
        To use MSAmanda on unix platforms, please install mono 
        (http://www.mono-project.com/download)

    '''
    uc = ursgal.UController(
        profile='LTQ XL low res',
        params={
            'database': os.path.join(
                os.pardir,
                'example_data',
                'BSA.fasta'
            ),
            'modifications': [
                'M,opt,any,Oxidation',        # Met oxidation
                'C,fix,any,Carbamidomethyl',  # Carbamidomethylation
                '*,opt,Prot-N-term,Acetyl',    # N-Acteylation
            ],
            # 'peptide_mapper_class_version' : 'UPeptideMapper_v2',
        }
    )

    if sys.maxsize > 2 ** 32:
        xtandem = 'xtandem_vengeance'
    else:
        xtandem = 'xtandem_sledgehammer'

    engine_list = [
        'omssa',
        xtandem,
        'msgfplus_v2016_09_16',
    ]

    mzML_file = os.path.join(
        os.pardir,
        'example_data',
        'BSA_simple_example_search',
        'BSA1.mzML'
    )
    if os.path.exists(mzML_file) is False:
        uc.params['http_url'] = 'http://sourceforge.net/p/open-ms/code/HEAD/tree/OpenMS/share/OpenMS/examples/BSA/BSA1.mzML?format=raw'
        uc.params['http_output_folder'] = os.path.dirname(mzML_file)
        uc.fetch_file(
            engine='get_http_files_1_0_0',
        )
        try:
            shutil.move(
                '{0}?format=raw'.format(mzML_file),
                mzML_file
            )
        except:
            shutil.move(
                '{0}format=raw'.format(mzML_file),
                mzML_file
            )

    unified_file_list = []

    for engine in engine_list:
        unified_search_result_file = uc.search(
            input_file=mzML_file,
            engine=engine,
            force=False
        )
        unified_file_list.append(unified_search_result_file)

    uc.visualize(
        input_files=unified_file_list,
        engine='venndiagram_1_1_0',
    )
    return


if __name__ == '__main__':
    main()


#!/usr/bin/env python
import ursgal
import os
from collections import defaultdict as ddict
import csv
import itertools
import sys

class pipi_1_4_5(ursgal.UNode):
    """
    Unode for PIPI: PTM-Invariant Peptide Identification
    For furhter information see: http://bioinformatics.ust.hk/pipi.html

    Note:
        Please download and extract PIPI manually from 
        http://bioinformatics.ust.hk/pipi.html

    Reference:
    Yu, F., Li, N., Yu, W. (2016) PIPI: PTM-Invariant Peptide Identification 
    Using Coding Method. J Prot Res 15(12)
    """

    META_INFO = {
        'edit_version': 1.00,
        'name': 'PIPI',
        'version': '1.4.5',
        'release_date': '2018-06-20',
        'utranslation_style': 'pipi_style_1',
        'input_extensions': ['.mgf', '.mzML', '.mzXML'],
        'output_extensions': ['.csv'],
        'create_own_folder': True,
        'in_development': False,
        'include_in_git': False,
        'distributable': True,
        'engine_type': {
            'protein_database_search_engine': True,
        },
        'engine': {
            'platform_independent': {
                'arc_independent': {
                    'exe': 'PIPI-1.4.5.jar',
                    'url': 'http://bioinformatics.ust.hk/pipi.html',
                    'zip_md5' : 'f28bf201a1d71da257e0035cf3a95823',
                    'additional_exe': [],
                },
            },
        },
        'citation':
        'Yu, F., Li, N., Yu, W. (2016) PIPI: PTM-Invariant '
            'Peptide Identification Using Coding Method. '
            'J Prot Res 15(12)'
    }

    def __init__(self, *args, **kwargs):
        super(pipi_1_4_5, self).__init__(*args, **kwargs)
        pass

    def preflight(self):
        '''
        Formatting the command line and writing the param input file via 
        self.params

        Returns:
                dict: self.params
        '''
        self.params['translations']['mgf_input_file'] = os.path.join(
            self.params['input_dir_path'],
            self.params['input_file']
        )
        self.params['translations']['output_file_incl_path'] = os.path.join(
            self.params['output_dir_path'],
            self.params['output_file']
        )
        self.param_file_name = os.path.join(
            self.params['translations']['output_file_incl_path'].strip('.csv')
            + 'pipi_params.def'
        )
        self.created_tmp_files.append(self.param_file_name)

        # pprint.pprint(self.params['translations']['_grouped_by_translated_key'])
        # pprint.pprint(self.params)
        # sys.exit(1)
        self.params_to_write = {
            'version': self.META_INFO['version'],
            'output_percolator_input': 1,
            'mod10': '0.0@X?',
            'pepNterm': 0.0,
            'pepCterm': 0.0,
            'proNterm': 0.0,
            'proCterm': 0.0,
        }
        symbols = ['~', '!', '%', '^', '&', '*', '+', '<', '>']
        for x, element in enumerate(symbols):
            self.params_to_write[
                'mod0{0}'.format(x + 1)
            ] = '0.0@X{0}'.format(element)
        for aa in 'ACDEFGHIKLMNOPQRSTUVWYnc':
            self.params_to_write[aa] = 0

        write_exclusion_list = [
            # 'label',
            '-Xmx',
            'frag_mass_tolerance_unit',
            'base_mz',
            # 'header_translations',
            # 'validation_score_field'
        ]

        # additional_15N_modifications = []
        # if self.params['translations']['label'] == '15N':
        #     for aminoacid, N15_Diff in ursgal.ukb.DICT_15N_DIFF.items():
        #         existing = False
        #         for mod_dict in self.params['mods']['fix']:
        #             if aminoacid == mod_dict['aa']:
        #                 mod_dict['mass'] += N15_Diff
        #                 mod_dict['name'] += '_15N_{0}'.format(aminoacid)
        #                 existing = True
        #         if existing == True:
        #             continue
        #         else:
        #             self.params['mods']['fix'].append(
        #                 {
        #                     'aa': aminoacid,
        #                     'mass': N15_Diff,
        #                     'name': '_15N_{0}'.format(aminoacid),
        #                     'pos': 'any',
        #                 }
        #             )

        if self.params['translations']['frag_mass_tolerance_unit'] == 'ppm':
            self.params['translations']['_grouped_by_translated_key']['ms2_tolerance'] = {
                'frag_mass_tolerance': ursgal.ucore.convert_ppm_to_dalton(
                    self.params['translations']['frag_mass_tolerance'],
                    base_mz=self.params['base_mz']
                )
            }

        for pipi_param_name in self.params['translations']['_grouped_by_translated_key'].keys():
            for ursgal_param_name, param_value in self.params['translations']['_grouped_by_translated_key'][pipi_param_name].items():
                if pipi_param_name in write_exclusion_list:
                    continue
                elif pipi_param_name == 'frag_clear_mz_range':
                    min_mz, max_mz = param_value
                    self.params_to_write['min_clear_mz'] = min_mz
                    self.params_to_write['max_clear_mz'] = max_mz
                elif type(pipi_param_name) is tuple:
                    for pn in pipi_param_name:
                        self.params_to_write[pn] = param_value
                elif pipi_param_name == 'enzyme':
                    enz, site, aa, inh = param_value.split(';')
                    self.params_to_write['enzyme'] = '{0} {1} {2} {3}'.format(
                        enz,
                        site,
                        aa,
                        inh
                    )
                elif pipi_param_name == 'modifications':
                    '''
                    # specify additional mass and amino acid. DO NOT change the last character.
                    # maximum number is 10
                    # empty entries must start with 0.0
                    mod01 = 79.966331@S~ # Phosphorylation
                    mod02 = 79.966331@T! # Phosphorylation
                    mod03 = 79.966331@Y% # Phosphorylation
                    mod04 = 42.010565@K^ # Acetylation
                    mod05 = 15.994915@M& # Oxidation
                    mod06 = 14.015650@K* # Methylation
                    mod07 = 0.0@X+
                    mod08 = 0.0@X<
                    mod09 = 0.0@X>
                    mod10 = 0.0@X?
                    '''
                    n = 0
                    for mod_dict in self.params['mods']['opt']:
                        '''
                        {'_id': 0,
                          'aa': '*',
                          'composition': {'C': 2, 'H': 2, 'O': 1},
                          'id': '1',
                          'mass': 42.010565,
                          'name': 'Acetyl',
                          'org': '*,opt,Prot-N-term,Acetyl',
                          'pos': 'Prot-N-term',
                          'unimod': True},
                        '''
                        if mod_dict['pos'] == 'Prot-N-term':
                            self.params_to_write['proNterm'] = mod_dict['mass']
                            continue
                        elif mod_dict['pos'] == 'Prot-C-term':
                            self.params_to_write['proCterm'] = mod_dict['mass']
                            continue
                        elif mod_dict['pos'] == 'N-term':
                            self.params_to_write['pepNterm'] = mod_dict['mass']
                            continue
                        elif mod_dict['pos'] == 'C-term':
                            self.params_to_write['pepCterm'] = mod_dict['mass']
                            continue
                        elif mod_dict['pos'] == 'any':
                            pass
                        else:
                            print(
                                '''
                            Unknown positional argument for given modification:
                            {0}
                            PIPI cannot deal with this, please use one of the follwing:
                            any, Prot-N-term, Prot-C-term, N-term, C-term
                            '''.format(mod_dict['org'])
                            )
                            sys.exit(1)
                        n += 1
                        assert n <= 10, '''
                        A maximum of 10 optional modifications is allowed in PIPI.
                        You specified more than 10.
                        '''
                        if n < 10:
                            mod_n = 'mod0{0}'.format(n)
                        elif n == 10:
                            mod_n = 'mod{0}'.format(n)
                        else:
                            print('''
                                A maximum of 10 optional modifications is allowed in PIPI.
                                You specified more than 10.
                            ''')
                            sys.exit(1)
                        self.params_to_write[mod_n] = '{0}@{1}{2}'.format(
                            mod_dict['mass'],
                            mod_dict['aa'],
                            symbols[n - 1]
                        )

                    for mod_dict in self.params['mods']['fix']:
                        if 'N-term' in mod_dict['pos']:
                            self.params_to_write['n'] = mod_dict['mass']
                        elif 'C-term' in mod_dict['pos']:
                            self.params_to_write['c'] = mod_dict['mass']
                        else:
                            self.params_to_write[
                                mod_dict['aa']] = mod_dict['mass']
                else:
                    self.params_to_write[pipi_param_name] = param_value
        self.write_params_file()

        self.params['command_list'] = [
            'java',
            '-Xmx{0}'.format(self.params['translations']
                             ['_grouped_by_translated_key']['-Xmx']['-xmx']),
            '-jar',
            self.exe,
            self.param_file_name,
            self.params['translations']['mgf_input_file']
        ]

        return self.params

    def postflight(self):
        # '''
        # Reads PIPI csv output and write final csv output file.

        # Adds:
        #     * Raw data location, since this can not be added later

        # '''
        pipi_fragger_header = [
            'scan_num',
            'peptide',
            'charge',
            'theo_mass',
            'exp_mass',
            'abs_ppm',
            'A_score',
            'protein_ID',
            'score',
            'delta_C_n',
            'other_PTM_patterns',
            'MGF_title',
            'labelling',
            'isotope_correction',
            'MS1_pearson_correlation_coefficient',
            # 'ptm_delta_score',
            # 'naive_q_value',
            # 'percolator_score',
            # 'posterior_error_prob',
            # 'percolator_q_value',
        ]

        translated_headers = []
        header_translations = self.UNODE_UPARAMS[
            'header_translations']['uvalue_style_translation']
        for original_header_key in pipi_fragger_header:
            ursgal_header_key = header_translations[original_header_key]
            translated_headers.append(ursgal_header_key)

        translated_headers += [
            'Raw data location',
            'Modifications',
            'Mass Difference',
        ]

        csv_writer = csv.DictWriter(
            open(self.params['translations']['output_file_incl_path'], 'w'),
            fieldnames=translated_headers
        )
        new_label = ""
        if self.params['label'] == '14N':
            new_label = 'N14'

        if self.params['label'] == '15N':
            new_label = 'N15'

        pipi_output = self.params['translations'][
            'mgf_input_file'] + '.{0}.pipi.csv'.format(new_label)
        csv_reader = csv.DictReader(
            open(pipi_output, 'r'),
            fieldnames=translated_headers,
        )
        # self.created_tmp_files.append(pipi_output)
        # self.created_tmp_files.append(pipi_output.replace('.pipi.csv', '.pipi.pep.xml'))
        # self.created_tmp_files.append(pipi_output.replace('.pipi.csv', '.input.temp'))

        csv_writer.writeheader()
        for n, line_dict in enumerate(csv_reader):
            if n == 0:
                continue
            # try:
            #     float(line_dict['PIPI:score'])
            # except:
            #     line_dict['Protein ID'] += line_dict['PIPI:score']
            #     line_dict['PIPI:score'] = line_dict['PIPI:delta_C_n']
            #     line_dict['PIPI:delta_C_n'] = line_dict['PIPI:other_PTM_patterns']
            #     line_dict['PIPI:other_PTM_patterns'] = line_dict['Spectrum Title']
            #     line_dict['Spectrum Title'] = line_dict['label']
            #     line_dict['label'] = line_dict['PIPI:isotope_correction']
            #     line_dict['PIPI:isotope_correction'] = line_dict['PIPI:MS1_pearson_correlation_coefficient']
            #     line_dict['PIPI:MS1_pearson_correlation_coefficient'] = line_dict['Raw data location']
            line_dict['Raw data location'] = os.path.abspath(
                self.params['translations']['mgf_input_file']
            )

        #     ############################################
        #     # all fixing here has to go into unify csv! #
        #     ############################################

            tmp_seq = ''
            tmp_mods = []
            line_dict['Sequence'] = line_dict['Sequence'].replace('n', '')
            for part in line_dict['Sequence'].split('('):
                if ')' in part:
                    mod, seq = part.split(')')
                    tmp_mods.append(
                        '{0}:{1}'.format(mod, len(tmp_seq))
                    )
                    tmp_seq += seq
                else:
                    tmp_seq += part
            tmp_seq = tmp_seq.replace('c', '')
            line_dict['Sequence'] = tmp_seq
            line_dict['Modifications'] = ';'.join(tmp_mods)
            csv_writer.writerow(line_dict)
        return

    def write_params_file(self):
        with open(self.param_file_name, 'w') as io:
            print('''# {version}
# First line is the parameter file version. Don't change it.
thread_num = {thread_num}
percolator_path = ursgal/resources/darwin/64bit/percolator_2_08/percolator_2_08

# Database
db = {db}
database_type = Others # Different types have different fasta header patterns. Available values: UniProt, SwissProt, TAIR, ITAG, RefSeq, Others
missed_cleavage = {missed_cleavage}
min_peptide_length = {min_peptide_length}
max_peptide_length = {max_peptide_length}
add_decoy = {add_decoy} # 0 = don't generate and search decoy sequences automatically; 1 = generate and search decoy sequences
add_contaminant = {add_contaminant} # 0 = don't add contaminant proteins automatically; 1 = add contaminant proteins

# Spectrum
ms_level = {ms_level}

# Tolerance
ms1_tolerance_unit = {ms1_tolerance_unit}
ms1_tolerance = {ms1_tolerance}
ms2_tolerance = {ms2_tolerance}
mz_bin_offset = {mz_bin_offset}
min_clear_mz = {min_clear_mz}
max_clear_mz = {max_clear_mz}

# Modification related
min_ptm_mass = {min_ptm_mass}
max_ptm_mass = {max_ptm_mass}

# Isotopic labelling strategy
15N = {15N} # 1: 15N. 0: 14N.

# considered modifications in generating tags
# specify additional mass and amino acid. DO NOT change the last character.
# maximum number is 10
# empty entries must start with 0.0
mod01 = {mod01}
mod02 = {mod02}
mod03 = {mod03}
mod04 = {mod04}
mod05 = {mod05}
mod06 = {mod06}
mod07 = {mod07}
mod08 = {mod08}
mod09 = {mod09}
mod10 = {mod10}

# considered peptide/protein N/C-terminal modifications in generating tags
# empty ones must with value 0.0
# the maximum number for each terminal is 9
Nterm = {pepNterm}
Cterm = {pepCterm}

# Fix modification
G = {G}
A = {A}
S = {S}
P = {P}
V = {V}
T = {T}
C = {C}
I = {I}
L = {L}
N = {N}
D = {D}
Q = {Q}
K = {K}
E = {E}
M = {M}
H = {H}
F = {F}
R = {R}
Y = {Y}
W = {W}
O = {O}
U = {U}
n = {n}
c = {c}

# Enzyme digestion specificities
# enzyme name    is cut from C-term      cleavage site   protection site
{enzyme}

# Do not change the following
output_percolator_input = 1
'''.format(
                **self.params_to_write),
                file=io
            )
        return



#!/usr/bin/env python
import pickle
import ursgal
import sys
import re
import os
from collections import defaultdict as ddict
from urllib.parse import unquote
import pprint
import re
import json

regexs_for_non_standard_mgfs = {
    'cz' : re.compile(r'''
        msmsid:F(?P<spec_id>[0-9]*),
        quan:(?P<quant>[0-9]*),
        start:(?P<rt_start_in_seconds>\d*\.\d+|\d+),
        end:(?P<rt_end_in_seconds>\d*\.\d+|\d+),
        survey:S(?P<survey>[0-9]*),
        parent:(?P<precursor_mz>\d*\.\d+|\d+),
    ''', re.VERBOSE)
}

def add_mascot_to_ursgal_history(file):
    """Add Mascot to history.

    Args:
        mapped_csv_search_results (str): Path to mapped search results.
    """
    mapped_json = '{0}.u.json'.format(file)
    iinfo, oinfo, params, history = json.load(open(mapped_json))
    mascot_missing = True
    for entry in history['history']:
        engine_type = entry['META_INFO'].get('engine_type', {})
        is_search_engine = engine_type.get('protein_database_search_engine', False)
        if is_search_engine and 'mascot' in entry['engine']:
            mascot_missing = False

    if mascot_missing:
        history['history'].insert(0,
            {
                "META_INFO": {
                  "engine_type": {
                    "protein_database_search_engine": True,
                    "by Hand" : True
                  }
                },
                "engine": "mascot_x_x_x"
          }
        )
        with open(mapped_json, "w") as d:
            json.dump([iinfo, oinfo, params, history], d)



def write_ursgal_pkl_from_mascot_dat(mascot_data_file):
    """
    Write ursgal pickle from mascot dat files

    Args:
        mascot_data_file (str): Path to mascot dat file for pickle generation.

    Returns:
        str: Path to the generated pickle file.
    """
    # rt_regex = re.compile(r'^.*rt\:(?P<rt>\d+\.\d+),.*')

    pattern = re.compile(r'Content-Type: application/x-Mascot; name="(?P<section>[a-z0-9]*)"')

    mascot_dat_base = os.path.basename(mascot_data_file)
    pkl_path = os.path.join(
        os.path.dirname(mascot_data_file),
        f'_ursgal_lookup.pkl'
    )

    sections = ddict(list)
    current_section = None
    with open(mascot_data_file) as mascot_file:
        for line in mascot_file:
            if line.strip() == '':
                continue
            match = pattern.match(line)
            if match is not None:
                current_section = match.group('section')
            else:
                sections[current_section].append(line.strip())

    for f in sections['parameters']:
        if f.startswith('FILE'):
            full_fname = f.split('=')[1]

            fname = os.path.splitext(os.path.basename(full_fname))[0]
            break
    dat_basename, ext = os.path.splitext(mascot_dat_base)

    if os.path.exists(pkl_path):
        t = pickle.load(open(pkl_path, 'rb'))
    else:
        t = {}
    if fname not in t:
        t.update(
            {
                fname : {
                    'rt_2_scan' : {},
                    'scan_2_rt' : {},
                    'unit'      : 'seconds',
                    'scan_2_mz' : {}
                }
            }
        )

    query_dict = {}
    for entry in sections:
        if entry is not None:
            if entry.startswith('query'):
                for key in sections[entry]:
                    if '=' in key:
                        k, v = key.split('=')
                        query_dict[k] = v
                title = query_dict['title']
                try:
                    path, spec_id, spec_id, charge = title.split('.')
                except:
                    path = os.path.basename(fname)
                    unqstring = unquote(title)
                    charge = int(query_dict['charge'].replace('+',''))
                    for _id, pattern in regexs_for_non_standard_mgfs.items():
                        m = pattern.match(unqstring)
                        if m is not None:
                            spec_id = int(m.group('spec_id'))

                            query_dict['rtinseconds'] = float(m.group('rt_start_in_seconds'))
                            query_dict['precursor_mz'] = m.group('precursor_mz')
                            break

                rt    = query_dict['rtinseconds']
                t[fname]['rt_2_scan'][float(rt)] = int(spec_id)
                t[fname]['scan_2_rt'][int(spec_id)] = float(rt)
                t[fname]['scan_2_mz'][int(spec_id)] = float(query_dict['precursor_mz'])
    with open(pkl_path, 'wb') as pkl:
        pickle.dump(t, pkl)
    return pkl_path


def main(input_file, database):
    """
    Usage:

    ./unify_mascot_dat.py <mascot dat file> <database>

    Note:
        Modifications are hard coded so please adjust accordingly
    """
    uc = ursgal.UController()
    uc.params['database'] = database
    uc.params['modifications'] = [
        'C,fix,any,Carbamidomethyl',
        '*,opt,Prot-N-term,Acetyl',
        'M,opt,any,Oxidation',
        '*,opt,N-term,TMT6plex',
        'K,fix,any,TMT6plex',
    ]
    uc.params['csv_filter_rules'] = [
        ['Sequence', 'contains_not', 'X'],
        # ['PEP', 'lte', 0.01],
    ]
    uc.params['decoy_tag'] = '###REV###'
    uc.scan_rt_lookup_path = write_ursgal_pkl_from_mascot_dat(input_file)
    converted_unfiltered = uc.execute_unode(
        input_file,
        engine='mascot_dat2csv'
    )
    converted = uc.execute_misc_engine(
        input_file=converted_unfiltered,
        engine='filter_csv'
    )
    mapped = uc.map_peptides_to_fasta(
        converted
    )
    add_mascot_to_ursgal_history(mapped)
    unified = uc.unify_csv(mapped)
    print(unified)

if __name__ == '__main__':
    if len(sys.argv) !=3:
        print(main.__doc__)
    else:
        main(sys.argv[1], sys.argv[2])



#!/usr/bin/env python3
# coding: latin1
import ursgal
from .percolator_3_2_1 import percolator_3_2_1


class percolator_3_4_0(percolator_3_2_1):
    """
    Percolator 3.4.0 UNode

    q-value and posterior error probability calculation
    by a semi-supervised learning algorithm that dynamically
    learns to separate target from decoy peptide-spectrum matches (PSMs)

    Reference:
    Matthew The, Michael J. MacCoss, William S. Noble, Lukas Käll "Fast and Accurate Protein False Discovery Rates on Large-Scale Proteomics Data Sets with Percolator 3.0"

    Noe:
    Please download Percolator corresponding to your OS from: https://github.com/percolator/percolator/releases
    """
    META_INFO = {
        'engine_type': {
            'controller': False,
            'converter': False,
            'validation_engine': True,
            'search_engine': False,
            'meta_engine': False
        },
        'edit_version': 1.00,
        'name': 'percolator',
        'version': '3.4.0',
        'release_date': None,
        'output_extensions': ['.csv'],
        'output_suffix': 'percolator_3_4_0_validated',
        'input_extensions': ['.csv'],
        'create_own_folder': False,
        'citation' : 'Matthew The, Michael J. MacCoss, William S. Noble, Lukas Kall' \
            'Fast and Accurate Protein False Discovery Rates on Large-Scale Proteomics Data Sets with Percolator 3.0',
        'include_in_git': False, # True for now, but upload dat shit later on
        'distributable': False,
        'group_psms': True,
        'in_development': False,
        'utranslation_style': 'percolator_style_1',
        'engine': {
            'darwin': {
                '64bit': {
                    'exe': 'percolator_3_4_0',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
            },
            'linux': {
                '64bit': {
                    'exe': 'percolator',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
            },
            'win32': {
                '64bit': {
                    'exe': 'percolator.exe',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
                '32bit': {
                    'exe': 'percolator.exe',
                    'url': '',
                    'zip_md5': '',
                    'additional_exe': [],
                },
            },
        }
    }

    def __init__(self, *args, **kwargs):
        super(percolator_3_4_0, self).__init__(*args, **kwargs)

